#!/usr/bin/env python3

import sys
from Bio import SeqIO
from Bio.Alphabet import generic_dna
from Bio.Seq import Seq
from Bio.Blast import NCBIXML
from Bio.Blast.Applications import NcbiblastpCommandline
import os
import pickle
import multiprocessing
import shutil

try:
	from utils import CommonFastaFunctions
except ImportError:
	from CHEWBBACA.utils import CommonFastaFunctions

#~ from cStringIO import StringIO
try:
	from StringIO import StringIO
except ImportError:
	from io import StringIO

def get_Short (gene,auxBar):
	blastPath='blastp'
	genesList=[str(gene)]

	pathtoDir=os.path.join(os.path.dirname(gene),"short")

	try:
		if not os.path.exists(pathtoDir):
			os.makedirs(pathtoDir)
	except Exception as e:
		pass
		#~ print (e)


	for gene in genesList:
		#~ print ("processing " +gene)

		pathtoDir=os.path.join(os.path.dirname(gene),"short")

		shortgene= os.path.join(os.path.dirname(gene),"short",os.path.basename(gene))
		shortgene= shortgene.replace(".fasta","_short.fasta")

		tempgene= os.path.join(os.path.dirname(shortgene),"temp",os.path.basename(gene).replace(".fasta",""))
		tempgeneProt= os.path.join(tempgene,os.path.basename(gene))
		tempgeneProt2= os.path.join(tempgene,os.path.basename(gene))
		tempgeneProt2= tempgeneProt2.replace(".fasta","2.fasta")

		tempgeneProtFasta=''
		tempgeneProt2Fasta=''
		shortfasta=''

		if not os.path.exists(tempgene):
			os.makedirs(tempgene)

		#~ gene_fp2 = HTSeq.FastaReader(gene)

		counter=0
		alleleI=0
		var={}

		geneScorePickle=shortgene+'_bsr.txt'
		selfscores=[]
		fasta_corrected=''
		total_alleles=0
		error_alleles=0
		corrected_alleles=0
		for allele in SeqIO.parse(gene, "fasta", generic_dna):
			total_alleles+=1
			try:
				translatedSequence,sortedSeq, originalSeq=translateSeq(str(allele.seq.upper()))

				if not originalSeq:
					fasta_corrected+='>'+str(allele.name)+'\n'+str(sortedSeq) + '\n'
					corrected_alleles+=1
				else:
					fasta_corrected+='>'+str(allele.name)+'\n'+str(str(allele.seq.upper())) + '\n'
				#~ alleleI=int(((allele.name).split("_"))[-1])
				alleleI=int(((allele.name).split("_"))[-1])
				if counter<1:

					#add first allele as short and calculate self bsr

					counter+=1

					shortfasta='>'+str(allele.name)+'\n'+str(allele.seq.upper()) + '\n'



					tempgeneProtFasta='>'+str(allele.name)+'\n'+str(translatedSequence) + '\n'

					Gene_Blast_DB_name = CommonFastaFunctions.Create_Blastdb_no_fasta(tempgeneProt, 1, True,tempgeneProtFasta)


					# --- get BLAST score ratio --- #

					cline = NcbiblastpCommandline(cmd=blastPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5,num_threads=1)
					out, err = cline(stdin=tempgeneProtFasta)
					psiblast_xml = StringIO(out)
					blast_records = NCBIXML.parse(psiblast_xml)


					allelescore=[]


					for blast_record in blast_records:

						for alignment in blast_record.alignments:

							for match in alignment.hsps:

								allelescore.append(int(match.score))

					selfbsr=float(allelescore[0])/float(allelescore[0])

					var[alleleI]= allelescore[0]

					selfscores.append(allelescore[0])

				else:

					#calculate selfbsr for each allele

					translatedSequence,sortedSeq, originalSeq=translateSeq(str(allele.seq.upper()))


					tempgeneProt2Fasta='>'+str(allele.name)+'\n'+str(translatedSequence) + '\n'


					Gene_Blast_DB_name = CommonFastaFunctions.Create_Blastdb_no_fasta(tempgeneProt2, 1, True,tempgeneProt2Fasta)


					# --- get BLAST score ratio --- #
					cline = NcbiblastpCommandline(cmd=blastPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5,num_threads=1)
					out, err = cline(stdin=tempgeneProt2Fasta)
					psiblast_xml = StringIO(out)
					blast_records = NCBIXML.parse(psiblast_xml)


					allelescore=[]


					for blast_record in blast_records:

						for alignment in blast_record.alignments:

							for match in alignment.hsps:

								allelescore.append(int(match.score))

					selfscore=allelescore[0]
					selfbsr=float(selfscore)/float(selfscore)


					#calculate bsr for the allele vs all previous alleles


					# --- get BLAST score ratio --- #
					cline = NcbiblastpCommandline(cmd=blastPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5,num_threads=1)
					out, err = cline(stdin=tempgeneProtFasta)
					psiblast_xml = StringIO(out)
					blast_records = NCBIXML.parse(psiblast_xml)



					allelescore=[]
					allelescoreId=[]

					bestbsr=0
					bestscore=0
					for blast_record in blast_records:

						for alignment in blast_record.alignments:

							for match in alignment.hsps:

								alleleMatchid=int((blast_record.query_id.split("_"))[-1])

								bsr=float(match.score)/float(selfscores[int(alleleMatchid-1)])
								if bsr>bestbsr and match.score>bestscore and bsr>=0.6:
									bestbsr=bsr
									bestscore=match.score

					if bestbsr>=0.6 and bestbsr<0.7:

						shortfasta+='>'+str(allele.name)+'\n'+str(allele.seq.upper()) + '\n'

						var[alleleI]= selfscore
						selfscores.append(selfscore)

						tempgeneProtFasta+='>'+str(allele.name)+'\n'+str(translatedSequence) + '\n'




			except Exception as e:
				#~ print ('Error on line {}'.format(sys.exc_info()[-1].tb_lineno))
				#print (str(allele.name)+" "+str(e))
				#print ("allele not translatable")
				error_alleles+=1

		#~ print ("processed " +gene)
		
		if error_alleles<total_alleles:

			with open(geneScorePickle,'wb') as f:
				pickle.dump(var, f)

			with open(shortgene,'w') as f:
				f.write(shortfasta)
		
		else:
			print ("ATTENTION!!!111 \n"+str(gene)+" has no correct aleles, the file will be removed!!")
			os.remove(gene)
		
		if corrected_alleles>=1 or error_alleles>=1:
			with open(gene,'w') as f:
				f.write(fasta_corrected)

		#print status bar
		if gene in auxBar:
			auxlen=len(auxBar)
			index=auxBar.index(gene)
			print ( "["+"="*index+">"+" "*(auxlen-index)+"] processed "+str(int((float(index)/auxlen)*100))+"%")



	return	True

def translateSeq(DNASeq):
	seq=DNASeq
	reversedSeq=False
	tableid=11
	originalSeq=True
	try:
		myseq= Seq(seq)
		protseq=Seq.translate(myseq, table=tableid,cds=True)
	except:
		reversedSeq=True
		originalSeq=False
		try:
			seq=reverseComplement(seq)
			myseq= Seq(seq)
			protseq=Seq.translate(myseq, table=tableid,cds=True)

		except:
			try:
				seq=seq[::-1]
				myseq= Seq(seq)
				protseq=Seq.translate(myseq, table=tableid,cds=True)
			except:
				reversedSeq=False
				try:
					seq=seq[::-1]
					seq=reverseComplement(seq)
					myseq= Seq(seq)
					protseq=Seq.translate(myseq, table=tableid,cds=True)
				except Exception as e:
					#~ print ("translation error")
					#~ print (e)
					raise
	return protseq, seq, originalSeq

def reverseComplement(strDNA):

	basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
	strDNArevC = ''
	for l in strDNA:

		strDNArevC += basecomplement[l]

	return strDNArevC[::-1]

def check_if_list_or_folder(folder_or_list):
	list_files = []
	# check if given a list of genomes paths or a folder to create schema
	try:
		list_files=[]
		gene_fp = open( folder_or_list, 'r')
		for gene in gene_fp:
			gene = gene.strip()
			list_files.append(gene)
		#~ f = open(folder_or_list, 'r')
		#~ f.close()
		#~ list_files = folder_or_list
	except IOError:

		for gene in os.listdir(folder_or_list):
			if not gene.endswith(".fasta"):
				continue
			try:
				genepath = os.path.join(folder_or_list, gene)
				
				if os.path.isdir(genepath):
					continue
				
				for allele in SeqIO.parse(genepath, "fasta", generic_dna):
					break
				list_files.append(os.path.abspath(genepath))
			except Exception as e:
				print (e)
				pass

	return list_files

def main(geneFiles,cpu2use):

	#~ parser = argparse.ArgumentParser(description="This program prepares a schema for a chewBBACA allele call, creating a short version of each fasta with only the most diverse alleles")
	#~ parser.add_argument('-i', nargs='?', type=str, help='List of genes files (list of fasta files)', required=True)
	#~ parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu', required=False, default=1)
	#~
	#~ 
	#~ args = parser.parse_args()
	#~ 
	#~ geneFiles = args.i
	#~ cpu2use = args.cpu

	listGenes=[]
	#~ gene_fp = open( geneFiles, 'r')
	#~ for gene in gene_fp:
	#~ gene = gene.rstrip('\n')
	#~ listGenes.append(gene)
	#~ gene_fp.close()	



	listGenes=check_if_list_or_folder(geneFiles)

	#test down bar
	auxBar=[]
	step=int((len(listGenes))/10)+1
	counter=0
	while counter < len(listGenes):
		auxBar.append(listGenes[counter])
		counter+=step


	tempFolder=''

	pool = multiprocessing.Pool(cpu2use)
	for gene in listGenes:

		pool.apply_async(get_Short,args=[str(gene),auxBar])
		tempFolder= os.path.join(os.path.dirname(gene),"short","temp")
	pool.close()
	pool.join()

	try:
		shutil.rmtree(tempFolder)
	except:
		pass




if __name__ == "__main__":
	main()



#!/usr/bin/env python3

import argparse
import csv
from collections import defaultdict

def main(profile1,profile2,outputFile):
	#~ parser = argparse.ArgumentParser(
	#~ description="This program joins two profiles with the ")
	#~ parser.add_argument('-p1', nargs='?', type=str, help='profile 1', required=True)
	#~ parser.add_argument('-p2', nargs='?', type=str, help='profile 2', required=True)
	#~ parser.add_argument('-o', nargs='?', type=str, help='outut file name', required=True)
	#~
	#~ args = parser.parse_args()
	#~
	#~ profile1 = args.p1
	#~ profile2 = args.p2
	#~ outputFile = args.o


	dictaux= defaultdict(list)

	print ("reading profile 1")
	with open(profile1) as csvfile:
		reader = csv.reader(csvfile, delimiter='\t')
		headers = next(reader)

		len1=0
		for row in reader:
			i=0
			for elem in row:
				#~ print (elem)
				dictaux[headers[i]].append(elem)
				i+=1
			len1+=1

	print ("reading profile 2")
	with open(profile2) as csvfile:
		reader = csv.reader(csvfile, delimiter='\t')
		headers = next(reader)

		len2=0
		for row in reader:
			i=0
			for elem in row:
				#~ print (elem)
				dictaux[headers[i]].append(elem)
				i+=1
			len2+=1

	listheaders2include=[]
	lists2print=[]
	for header in headers:
		aux= dictaux[header]
		if len(aux)==len1+len2:
			listheaders2include.append(header)
	i=0

	print ("building new profile")
	while i<len1+len2:
		aux=[]
		for header in listheaders2include:
			aux.append(dictaux[header][i])
		i+=1
		lists2print.append(aux)

	newProfileStr=('\t'.join(map(str, listheaders2include)))+"\n"
	for elem in lists2print:
		newProfileStr += ('\t'.join(map(str, elem)))+"\n"

	with open(outputFile, "w") as f:
		f.write(newProfileStr)


	print ("Done")


if __name__ == "__main__":
	main()



#!/usr/bin/env python3
from Bio import SeqIO
from Bio.Alphabet import generic_dna
from Bio.Seq import Seq
import os
import argparse
import json
from Bio.Align.Applications import MafftCommandline
from Bio.Align.Applications import ClustalwCommandline
import multiprocessing
import copy

def reverseComplement(strDNA):

    basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    strDNArevC = ''
    for l in strDNA:

        strDNArevC += basecomplement[l]

    return strDNArevC[::-1]

def translateSeq(DNASeq,transTable):
    seq=DNASeq
    tableid=transTable
    reversedSeq=False
    try:
        myseq= Seq(seq)
        protseq=Seq.translate(myseq, table=tableid,cds=True)
    except:
        reversedSeq=True
        try:
            seq=reverseComplement(seq)
            myseq= Seq(seq)
            protseq=Seq.translate(myseq, table=tableid,cds=True)

        except:
            try:
                seq=seq[::-1]
                myseq= Seq(seq)
                protseq=Seq.translate(myseq, table=tableid,cds=True)
            except:
                reversedSeq=False
                try:
                    seq=seq[::-1]
                    seq=reverseComplement(seq)
                    myseq= Seq(seq)
                    protseq=Seq.translate(myseq, table=tableid,cds=True)
                except Exception as e:

                    raise ValueError(e)
    return protseq,seq,reversedSeq


def call_mafft(path_to_save,genefile):

    try:
        print("maffting "+os.path.basename(genefile))
        mafft_cline = MafftCommandline(input=genefile)
        stdout, stderr = mafft_cline()
        with open(path_to_save, "w") as handle:
            handle.write(stdout)
        return True

    except Exception as e:
        print(e)
        return False
def call_clustalw(genefile,htmlpath):

    try:
        print("clustaling "+os.path.basename(genefile))
        inputfile=os.path.join(htmlpath,(os.path.basename(genefile)).replace(".fasta","_aligned.fasta"))
        clw_cline = ClustalwCommandline("clustalw2", infile=inputfile, tree=True)
        clw_cline()

        return True

    except Exception as e:
        print(e)
        return False
#~ def main():
#~ 
    #~ parser = argparse.ArgumentParser(description="This program analyses cds")
    #~ parser.add_argument('-i', nargs='?', type=str, help='list genes', required=True)
    #~ parser.add_argument('-r', nargs='?', type=bool, help='Return values', required=False)
#~ 
    #~ args=parser.parse_args()
#~ 
#~ 
    #~ genes = args.i
#~ 
    #~ try:
        #~ ReturnValues=bool(args.r)
    #~ except:
        #~ ReturnValues=False
        #~ pass
#~ 
    #~ analyzeCDS(genes,ReturnValues)

def main(genes,transTable,ReturnValues,outputpath,cpu,skipClustalMafft):

    gene_fp = open( genes, 'r')

    stopc=0
    notStart=0
    notMultiple=0
    totalalleles=0

    htmlgenespath=os.path.join(outputpath,"genes_html/")

    if not os.path.exists(htmlgenespath):
        os.makedirs(htmlgenespath)

    listgenes=[]
    for gene in gene_fp:

        gene = gene.rstrip('\n')
        gene = gene.rstrip('\r')
        listgenes.append(gene)
    statsPerGene={}


    if not skipClustalMafft:
        pool = multiprocessing.Pool(cpu)

        for gene in listgenes:

            gene = gene.rstrip('\n')
            gene = gene.rstrip('\r')

            alignFileName=os.path.join(htmlgenespath,(os.path.basename(gene)).replace(".fasta","_aligned.fasta"))

            pool.apply_async(call_mafft,args=[alignFileName,gene])

        pool.close()
        pool.join()

        pool = multiprocessing.Pool(cpu)

        for gene in listgenes:

            gene = gene.rstrip('\n')
            gene = gene.rstrip('\r')

            alignFileName=os.path.join(htmlgenespath,(os.path.basename(gene)).replace(".fasta","_aligned.fasta"))

            pool.apply_async(call_clustalw,args=[gene,htmlgenespath])

        pool.close()
        pool.join()

    toPrintCDSStats="Locus\tFrameshift\tNo Start or Stop\tMore than 1 Stop\t Other\n"
    for gene in listgenes:

        gene = gene.rstrip('\n')
        gene = gene.rstrip('\r')

        alignFileName=os.path.join(htmlgenespath,(os.path.basename(gene)).replace(".fasta","_aligned.fasta"))


        listStopc=[]
        listnotStart=[]
        listnotMultiple=[]
        listOther=[]

        print("processing : "+str(os.path.basename(gene)))

        k=0

        multiple=True
        #gene_fp2 = HTSeq.FastaReader(gene)

        alleleSizes=[]
        alleleNames=[]
        alleleSizesNotMultiple=[]
        alleleSizesNotMultipleNames=[]
        alleleSizesTransError=[]
        alleleSizesTransErrorNames=[]
        # translate each allele and report the error if unable to translate
        for allele in SeqIO.parse(gene, "fasta", generic_dna):

            k+=1
            realAlleleID=allele.id.split("_")[-1]
            # if allele is not multiple of 3 it's useless to try to translate
            if (len(str(allele.seq)) % 3 != 0):
                multiple=False
                listnotMultiple.append(realAlleleID)
                alleleSizesNotMultipleNames.append(realAlleleID)
                alleleSizesNotMultiple.append(len(str(allele.seq)))
                pass
            else:
                try:
                    protseq,seq,reversedSeq=translateSeq(allele.seq, transTable)
                    alleleNames.append(realAlleleID)
                    alleleSizes.append(len(str(allele.seq)))

                except Exception as err:
                    if "Extra in frame stop codon found" in str(err):
                        stopc+=1
                        listStopc.append(str(k))
                    elif "is not a start codon" in str(err):
                        notStart+=1
                        listnotStart.append(str(k))
                    elif "is not a stop codon" in str(err):
                        notStart+=1
                        listnotStart.append(realAlleleID)
                    else:
                        listOther.append(realAlleleID)
                        print(err)

                    alleleSizesTransErrorNames.append(realAlleleID)
                    alleleSizesTransError.append(len(str(allele.seq)))
                    #print "allele "+str(k)+" is not translating"
                    pass


        relpath=os.path.relpath(gene,outputpath)
        statsPerGene[relpath]=copy.deepcopy([listnotMultiple,listStopc,listnotStart,k])
        totalalleles+=k

        #create html per gene
        genename=(os.path.basename(gene)).split(".")
        genename.pop()
        genename=".".join(genename)
        with open(htmlgenespath+genename+".html", "w") as f:
            f.write("<meta charset='UTF-8'><!DOCTYPE html>\n<html>\n<head><script type='text/javascript' src='https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js'></script>\n")
            f.write("<script type='text/javascript' src='https://ajax.googleapis.com/ajax/libs/jquery/2.1.3/jquery.min.js'></script>")
            f.write("""<!-- Latest compiled and minified JavaScript -->
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/js/bootstrap.min.js" integrity="sha384-0mSbJDEHialfmuBBQP6A4Qrprq5OVfW37PRR3j5ELqxss1yVqOtnepnHVP9aJ7xS" crossorigin="anonymous"></script>""")
            f.write("""<style type="text/css">
      body {
        padding-top: 10px;
        padding-bottom: 60px;
        padding-left: 20px;
      }

      /* Custom container */
      .container {
        margin: 0 auto;
        max-width: 1000px;
      }
      .container > hr {
        margin: 60px 0;
      }

      /* Main marketing message and sign up button */
      .jumbotron {
        margin: 40px 0;
        text-align: center;
        padding-top: 30px;
        padding-bottom: 30px;
      }
      .jumbotron h1 {
        font-size: 100px;
        line-height: 1;
      }
      
      phylocanvas {
          width: 100%;
          height: 30em;
        }
        </style>
      <!-- Latest compiled and minified CSS -->
      <script type="application/javascript" src="https://cdn.rawgit.com/phylocanvas/phylocanvas-quickstart/v2.3.0/phylocanvas-quickstart.js"></script>
            <script src="https://cdn.bio.sh/msa/latest/msa.min.gz.js"></script>
            <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
        
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css" integrity="sha384-1q8mTJOASx8j1Au+a5WDVnPi2lkFfwwEAa8hDDdjZlpLegxhjVME1fgjWPGmkzs7" crossorigin="anonymous">""")

            f.write("""<div class="jumbotron">
                      <h2>Analysis of the locus """+(os.path.basename(gene)).replace(".fasta","")+"""</h2>
                      <p>Explore the analysis by clicking the analysis buttons</p></div>""")
            relpath=os.path.relpath(gene,htmlgenespath)
            #print relpath
            f.write("<input type=button onClick=window.open('"+relpath+"') value='Get fasta file'>\n<p></p>""")
            f.write("<input type=button onClick=window.open('"+os.path.join((os.path.relpath(alignFileName,alignFileName)),(os.path.basename(gene)).replace(".fasta","_aligned.fasta"))+"') value='Get msa file'>\n""")

            f.write("""<p>MSA alignement done using mafft and visualized using msaviewer</p>
                <p>Tree built with clustalw2 and visualized using phylocanvas</p>
                <div class='container'>
                    <div class='row'>
                        
                        <div class="col-sm-3">
                        <button id='button3' class="btn btn-info btn-block active">Allele Size plot</button>
                        </div>
                        <div class="col-sm-3">
                        <button id='button1' class="btn btn-info btn-block active">NJ tree</button>
                        </div>
                        <div class="col-sm-3">
                        <button id='button2' class="btn btn-info btn-block active">MSA alignment</button>
                        </div>
                    </div>
                </div>""")


            f.write("""</head>\n<body><p></p><div id='snippetDiv' style='display:none'></div> <p></p><div id='phylocanvas' style='display:none;border:solid'>
            <div id="pc-buttons">
                <input type="text" id="searchbox" onkeyup="search(this);">
              <button id="rectangular" class="btn btn-default btn-sm">Rectangular</button>
              <button id="circular" class="btn btn-info btn-sm">Circular</button>
              <button id="radial" class="btn btn-default btn-sm">Radial</button>
              <button id="diagonal" class="btn btn-default btn-sm">Diagonal</button>
              <button id="hierarchical" class="btn btn-default btn-sm">Hierarchical</button>
            </div>
            
            </div><div id='histdiv'></div>\n""")


            f.write("""
            
            <script type="application/javascript">
                        var tree;
                        function doPhylocanvas() {
                          tree = Phylocanvas.createTree('phylocanvas', {
                          // other options
                          contextMenu : [{
                            text: 'Normal Menu',
                            handler: 'triggerNormal',
                            internal: false,
                            leaf: false
                          }, {
                            text: 'Internal Menu',
                            handler: 'triggerInternal',
                            internal: true,
                            leaf: false
                          }, {
                            text: 'Save as PNG',
                            handler: 'exportCurrentTreeView',
                            internal: false,
                            leaf: false
                          }]
                          });
                          var treestr=$.ajax({
                            url: '"""+os.path.join((os.path.relpath(alignFileName,alignFileName)),(os.path.basename(gene)).replace(".fasta","_aligned.ph"))+"""',
                            async: false
                         }).responseText;
                         tree.setTreeType('circular');
                          
                          tree.load(treestr);
                        }  
                          function search (ele) {
                            if(ele.value !== ""){
                              someleaves=tree.findLeaves(ele.value);
                              for (index = 0, len = someleaves.length; index < len; ++index) {
                                    if (someleaves[index]== null){
                                    someleaves.selected = true;
                                    
                                    }
                                    else{
                                    someleaves[index].selected = true;
                                    }
                                    
                                }
                            }
                            else {
                              tree.branches.E.cascadeFlag('selected', false);
                              tree.branches.E.cascadeFlag('highlighted', false);
                              tree.draw();
                                }
                            }
                        
                        $(document).on('click','#pc-buttons .btn', {} ,function(e){
                            $('#pc-buttons .btn').removeClass('btn-info');
                            $('#pc-buttons .btn').addClass('btn-default');
                            $(this).addClass('btn-info');
                            tree.setTreeType(this.id);
                        });
                        
                        
                        
                      </script>
            <script>
            

                    var yourDiv = document.getElementById('snippetDiv');
                    var menuDiv = document.createElement('div');
                    var msaDiv = document.createElement('div');
                    yourDiv.appendChild(menuDiv);
                    yourDiv.appendChild(msaDiv);

                    /* global yourDiv */
                    var opts = {
                      el: msaDiv,
                      importURL: '"""+os.path.join((os.path.relpath(alignFileName,alignFileName)),(os.path.basename(gene)).replace(".fasta","_aligned.fasta"))+"""',
                      colorscheme: {"scheme": "nucleotide"},
                    };
                    
                    opts.vis = {
                      scaleslider:true,
                    };
                    
                    var m = msa(opts);
                    m.render()

                    var defMenu = new msa.menu.defaultmenu({
                      el: menuDiv,
                      msa: m
                    });
                    defMenu.render();
                    
                    function addColumnFilter(menu){    
                        var msa = menu.msa;
                        var hidden = [];    threshold = 100 / 100;
                       var maxLen = msa.seqs.getMaxLength();
                       var hidden = [];
                       // TODO: cache this value
                       var conserv = msa.g.stats.scale(msa.g.stats.conservation());

                       var end = maxLen - 1;
                       for (var i = 0; 0 < end ? i <= end : i >= end; 0 < end ? i++ : i--) {
                         if (conserv[i] == threshold) {
                             hidden.push(i);
                         }
                       }
                       return msa.g.columns.set("hidden", hidden);}
                      
                     function addColumnFilter2(menu){    
                        var msa = menu.msa;
                        var seqToHideStr = prompt("Enter Seq to show", 'eg allele1,allele2');
                        var seqToHide = seqToHideStr.split(',');

                       // TODO: cache this value
                       var i = 0;
                       var len = (m.seqs.models.length)-1

                       for (; len >= i; ) {
                            var auxSeq=msa.seqs.models[len]
                            if ( $.inArray(auxSeq.attributes.name,seqToHide) < 0) {
                                msa.seqs.remove(msa.seqs.at(len));

                                }
                            len--;
                            }
                    
                       }
                      
                     $($('div').find('ul')[2]).prepend('<li id="removePoly">Hide Non Polymorphic Sites</li>');    $('#removePoly').click(function(){
                            addColumnFilter(defMenu);
                        });
                    
                    $($('div').find('ul')[2]).prepend('<li id="removeSpecSeq">Show specific set of seqs</li>');    $('#removeSpecSeq').click(function(){
                            addColumnFilter2(defMenu);
                        });	
                        
                    delete defMenu.views['10_import'];
                    </script>""")
            f.write("""<script type="text/javascript">
                    $("#button2").click(function(){
                      $("#snippetDiv").css({"display":"block"});
                      $("#histdiv").css({"display":"none"});
                      $("#phylocanvas").css({"display":"none"});
                    }); 
                    </script>
        
                    <script type="text/javascript">
                        $("#button3").click(function(){
                          $("#histdiv").css({"display":"block"});
                          $("#phylocanvas").css({"display":"none"});
                          $("#snippetDiv").css({"display":"none"});
                        }); 
                        </script>
            
                    <script type="text/javascript">
                        $("#button1").click(function(){
                          $("#phylocanvas").css({"display":"block"});
                          if ($("#phylocanvas__canvas").length < 1){
                            doPhylocanvas();}
                          $("#snippetDiv").css({"display":"none"});
                          $("#histdiv").css({"display":"none"});
                        }); 
                        </script>""")



            alleleSizes=str(json.dumps([alleleNames,alleleSizes]))
            alleleSizesNotMultiple=str(json.dumps([alleleSizesNotMultipleNames,alleleSizesNotMultiple]))
            alleleSizesTransError=str(json.dumps([alleleSizesTransErrorNames,alleleSizesTransError]))

            f.write("<script type='text/javascript'>var jsonScat1 ="+alleleSizes+""";
            
            
                        
                        var listTrace=[];
                        var trace1 = {
                                    x: jsonScat1[0],
                                      y: jsonScat1[1],
                                      text: jsonScat1[0],
                                      name: 'CDS alleles',
                                      mode: 'markers',
                                      type: 'scattergl'
                                    };
                        listTrace.push(trace1);""")
            if 	len(alleleSizesNotMultipleNames)>0:


                f.write("var jsonScat2 ="+alleleSizesNotMultiple+""";
                
                        var trace2 = {
                                    x: jsonScat2[0],
                                      y: jsonScat2[1],
                                      text: jsonScat2[0],
                                      name: 'Alleles not multiple of 3',
                                      mode: 'markers',
                                      type: 'scattergl'
                                    };
                        listTrace.push(trace2);
                        """)
            if 	len(alleleSizesTransErrorNames)>0:

                f.write("var jsonScat3 ="+alleleSizesTransError+""";
                            var trace3 = {
                                    x: jsonScat3[0],
                                      y: jsonScat3[1],
                                      text: jsonScat3[0],
                                      name: 'Non translatable alleles',
                                      mode: 'markers',
                                      type: 'scattergl'
                                    };
                        listTrace.push(trace3);
                        """)

            f.write("""var layout = {
                                      title: 'Allele size scatter plot',
                                      yaxis: {title: "DNA bp allele length"},
                                      xaxis: {
                                                title: "Allele number",
                                              },
                                    };
                        Plotly.newPlot('histdiv', listTrace,layout);
            
            
                    </script>"""+"</body></html>")


        if not(not listnotMultiple and not listnotStart and not listStopc and not listOther):
            if not listnotMultiple:
                listnotMultiple.append("-")
            if not listnotStart:
                listnotStart.append("-")
            if not listStopc:
                listStopc.append("-")
            if not listOther:
                listOther.append("-")
            toPrintCDSStats+= os.path.basename(gene)+"\t"+','.join(listnotMultiple)+"\t"+','.join(listnotStart)+"\t"+','.join(listStopc)+"\t"+','.join(listOther)+"\n"
    #print str(stopc) + " alleles have stop codons inside"
    #print str(notStart) + " alleles don't have start codons"
    print("total of alleles : " + str(totalalleles))

    with open('non_cds_alleles.tsv', "w") as f:
        f.write(toPrintCDSStats)
    print(toPrintCDSStats)

    return statsPerGene

if __name__ == "__main__":
    main()




#!/usr/bin/env python3
from Bio import SeqIO
from Bio.Alphabet import generic_dna
from Bio.Seq import Seq
import os
import argparse
import time
import pickle
import shutil
import multiprocessing
import subprocess
try:
    from createschema import CreateSchema,runProdigal
except ImportError:
    from CHEWBBACA.createschema import CreateSchema,runProdigal


def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True

    return "Not found"


def checkGeneStrings(genome1, genome2, newName, basepath, cpu, blastp, createSchemaPath,verbose,bsr):

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function

    pathForTemp = os.path.join(basepath, newName)
    if not os.path.exists(pathForTemp):
        os.makedirs(pathForTemp)

    listOfGenomes = [genome1, genome2]

    dictprots = {}
    dictprotsLen = {}
    dictprotsName = {}
    newlistOfCDS = {}
    proteinsEqual = 0
    smallProteins = 0
    protid = 0
    genomeProts = ""
    genomeProtsTrans = ""

    try:

        for genomeFile in listOfGenomes:

            genomename = (os.path.basename(genomeFile)).split(".")
            genomename = genomename[0]

            listOfCDS = {}

            currentCDSDict = {}
            currentGenomeDict = {}
            filepath = os.path.join(basepath, str(os.path.basename(genomeFile)) + "_ORF.txt")
            newfilepath = os.path.join(basepath, str(newName))

            #g_fp = HTSeq.FastaReader(genomeFile)
            for contig in SeqIO.parse(genomeFile, "fasta", generic_dna):
                sequence = str(contig.seq.upper())
                currentGenomeDict[contig.id] = sequence

            # after the first iteration, genomes are already defined by their cds and no longer have a cds dictionary pickle file
            try:
                with open(filepath, 'rb') as f:
                    currentCDSDict = pickle.load(f)
            except:
                for k, v in currentGenomeDict.items():
                    currentCDSDict[k] = [[v]]

            j = 0

            counter = 0
            tsvProtidGenome=""

            for contigTag, value in currentCDSDict.items():

                for protein in value:
                    protid += 1


                    # at first iteration we use the genome file and after a cds only multifasta file
                    try:
                        seq = currentGenomeDict[contigTag][protein[0]:protein[1]].upper()
                        aux=[genomename,contigTag,str(protein[0]),str(protein[1]),str(protid)]
                        tsvProtidGenome+="\n"+'\t'.join(aux)

                    except Exception as e:

                        seq = str(protein[0])

                    try:
                        protseq, orderedSeq = translateSeq(seq)
                        lengthofProt = len(str(protseq))
                    except:
                        verboseprint( str(genome1) + " " + str(genome2))
                        pass
                    # check if any protein with size on dict

                    try:

                        if len(str(protseq)) < 67:
                            smallProteins += 1
                            pass

                        elif dictprotsLen[lengthofProt]:
                            proteinFound = False
                            listproteinsid = dictprotsLen[lengthofProt]
                            for elem in listproteinsid:

                                if protseq == dictprots[elem]:
                                    proteinFound = True
                                    proteinsEqual += 1
                                    break

                            if not proteinFound:
                                dictprotsLen[lengthofProt].append(protid)
                                dictprots[protid] = protseq
                                newlistOfCDS[protid] = orderedSeq
                                try:
                                    protein[1]
                                    idstr = ">" + str(genomename) + "|protein" + str(protid)
                                    idstr2 = ">" + str(genomename) + "|protein" + str(protid)
                                except:
                                    idstr = ">" + str(contigTag)
                                    idstr2 = ">" + str((contigTag.split(" "))[0])
                                genomeProts += idstr + "\n"
                                genomeProtsTrans += idstr2 + "\n"
                                genomeProts += str(orderedSeq) + "\n"
                                genomeProtsTrans += str(protseq) + "\n"
                                dictprotsName[protid] = idstr2
                    except Exception as e:
                        try:
                            dictprotsLen[lengthofProt] = [protid]
                            dictprots[protid] = protseq
                            newlistOfCDS[protid] = orderedSeq
                            try:
                                protein[1]
                                idstr = ">" + str(genomename) + "|protein" + str(protid)
                                idstr2 = ">" + str(genomename) + "|protein" + str(protid)

                            except:
                                idstr = ">" + str(contigTag)
                                idstr2 = ">" + str((contigTag.split(" "))[0])

                            genomeProts += idstr + "\n"
                            genomeProtsTrans += idstr2 + "\n"
                            genomeProts += str(orderedSeq) + "\n"
                            genomeProtsTrans += str(protseq) + "\n"
                            dictprotsName[protid] = idstr2
                        except:
                            pass

                    else:
                        pass

            listOfCDS = ''
            currentGenomeDict = ''
            currentCDSDict = ''
            with open("proteinID_Genome.tsv", 'a') as f:
                f.write(tsvProtidGenome)

        verboseprint( "Checked equal proteins for: " + str(genome1) + " " + str(genome2))
        verboseprint( "Starting with a total of loci: " + str(protid))
        verboseprint( "equal proteins : " + str(proteinsEqual))
        verboseprint( "small proteins : " + str(smallProteins))

        fastaFile = os.path.join(pathForTemp, newName + ".fasta")
        with open(fastaFile, 'a') as f:
            f.write(genomeProts)

        newlistOfCDS = {}
        genomeProtsTrans = ''
        genomeProts = ''

        # check if any protein is substring of a larger, mantaining the larger one
        # ordering the sequences by length, larger ones first
        auxlist = dictprotsLen.keys()
        auxlist = sorted(auxlist, key=int)
        auxlist = auxlist[::-1]

        finalProtDict = {}
        genomeProtsTrans = ''
        auxprotlist = []
        contained = 0
        finalnumber = 0
        verboseprint( "Looking for contained proteins in : " + str(genome1) + " " + str(genome2))
        counter = 0

        for elem in auxlist:
            counter += 1

            for protid in dictprotsLen[elem]:
                str2 = str(dictprots[protid])

                try:
                    auxprotlist[0]
                    for elem2 in auxprotlist:
                        isContained = False
                        if len(elem2) < len(str2):
                            break
                        else:
                            if str2 in elem2:
                                isContained = True
                                contained += 1
                                break
                    if not isContained:
                        str1 = dictprotsName[protid]
                        genomeProtsTrans += str1 + "\n" + str2 + "\n"
                        finalnumber += 1
                        auxprotlist.append(str2)
                except Exception as e:
                    str1 = dictprotsName[protid]
                    genomeProtsTrans += str1 + "\n" + str2 + "\n"
                    finalnumber += 1
                    auxprotlist.append(str2)

        auxprotlist = []
        dictprots = {}
        dictprotsLen = {}
        dictprotsName = {}

        verboseprint( "number of contained proteins : " + str(contained))
        verboseprint( "total of loci to blast : " + str(finalnumber))

        proteinFile = os.path.join(pathForTemp, newName + "_proteins.fasta")

        with open(proteinFile, 'a') as f:
            f.write(genomeProtsTrans)
        genomeProtsTrans = ''

        # run createschema for the final protogenome
        verboseprint( "running blast will use this number of cpu: " + str(cpu))
        CreateSchema.main(fastaFile,200,cpu,proteinFile,fastaFile,blastp,bsr,verbose)
        #~ proc = subprocess.Popen(
        #~ [createSchemaPath, '-i', fastaFile, '-l', "200", '--cpu', str(cpu), '-p', proteinFile, '-o', fastaFile,
        #~ "-b", blastp], stdout=subprocess.PIPE)
        #~ p_status = proc.wait()
        verboseprint( "finished blast")

        os.remove(proteinFile)


    except Exception as e:
        verboseprint( e )
        return e

    return True


def reverseComplement(strDNA):
    basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    strDNArevC = ''
    for l in strDNA:
        strDNArevC += basecomplement[l]

    return strDNArevC[::-1]


def translateSeq(DNASeq):
    seq = DNASeq
    tableid = 11

    #look for ambiguous bases
    try:
        reverseComplement(seq)
    except:
        raise
    try:
        myseq = Seq(seq)
        protseq = Seq.translate(myseq, table=tableid, cds=True)
    except:
        try:
            seq = reverseComplement(seq)
            myseq = Seq(seq)
            protseq = Seq.translate(myseq, table=tableid, cds=True)

        except:
            try:
                seq = seq[::-1]
                myseq = Seq(seq)
                protseq = Seq.translate(myseq, table=tableid, cds=True)
            except:
                try:
                    seq = seq[::-1]
                    seq = reverseComplement(seq)
                    myseq = Seq(seq)
                    protseq = Seq.translate(myseq, table=tableid, cds=True)
                except Exception as e:
                    #print "translation error"
                    #print e
                    raise

    return protseq, seq


def call_proc(cmd):
    p = subprocess.Popen(args=cmd)
    out, err = p.communicate()
    return p


# ================================================ MAIN ================================================ #

def main(genomeFiles,cpuToUse,outputFile,bsr,BlastpPath,min_length,verbose,chosenTrainingFile,inputCDS):
    #~ parser = argparse.ArgumentParser(description="This program call alleles for a set of genomes provided a schema")
    #~ parser.add_argument('-i', nargs='?', type=str, help='List of genome files (list of fasta files)', required=True)
    #~ parser.add_argument('-o', nargs='?', type=str, help="Name of the output files", required=True)
    #~ parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus, if over the maximum uses maximum -2",
    #~ required=True)
    #~ parser.add_argument('-b', nargs='?', type=str, help="BLAST full path", required=False, default='blastp')
    #~ parser.add_argument('--bsr', nargs='?', type=float, help="minimum BSR similarity", required=False, default=0.6)
    #~ parser.add_argument('-l', nargs='?', type=int, help="minimum bp locus lenght", required=False, default=200)
    #~ parser.add_argument('-t', nargs='?', type=str, help="taxon", required=False, default=False)
    #~ parser.add_argument('--ptf', nargs='?', type=str, help="provide own training file path", required=False, default=False)
    #~ parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
    #~ default=False)
    #~
    #~ args = parser.parse_args()
    #~
    #~ genomeFiles = args.i
    #~ cpuToUse = args.cpu
    #~ outputFile = args.o
    #~ BlastpPath = args.b
    #~ bsr = args.bsr
    #~ chosenTaxon = args.t
    #~ chosenTrainingFile = args.ptf
    #~ verbose = args.verbose
    #~ min_length = args.l

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function


    # avoid user to run the script with all cores available, could impossibilitate any usage when running on a laptop
    if cpuToUse > multiprocessing.cpu_count() - 2:
        print("Warning, you are close to use all your cpus, if you are using a laptop you may be uncapable to perform any action")

    taxonList = {'Campylobacter jejuni': 'trained_campyJejuni.trn',
                 'Acinetobacter baumannii': 'trained_acinetoBaumannii.trn',
                 'Streptococcus agalactiae': 'trained_strepAgalactiae.trn',
                 'Haemophilus influenzae': 'trained_haemoInfluenzae_A.trn',
                 'Yersinia enterocolitica': 'trained_yersiniaEnterocolitica.trn',
                 'Escherichia coli': 'trained_eColi.trn',
                 'Enterococcus faecium': 'trained_enteroFaecium.trn',
                 'Staphylococcus haemolyticus': 'trained_staphHaemolyticus.trn',
                 'Salmonella enterica': 'trained_salmonellaEnterica_enteritidis.trn',
                 'Staphylococcus aureus': 'trained_StaphylococcusAureus.trn',
                 'Streptococcus pneumoniae': 'trained_strepPneumoniae.trn'
                 }
    #~ if isinstance(chosenTaxon, str):
        #~ trainingFolderPAth = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'TrainingFiles4Prodigal'))
        #~ try:
            #~ chosenTaxon = os.path.join(trainingFolderPAth, taxonList[chosenTaxon])
#~ 
            #~ if os.path.isfile(chosenTaxon):
                #~ print("will use this training file : " + chosenTaxon)
            #~ else:
                #~ print("training file don't exist")
                #~ print(chosenTaxon)
                #~ return "retry"
        #~ except:
            #~ print("Your chosen taxon "+chosenTaxon+" is not attributed, select one from:")
            #~ for elem in taxonList.keys():
                #~ print(elem)
            #~ return "retry"

    if isinstance(chosenTrainingFile, str):
        trainingFolderPAth = os.path.abspath(chosenTrainingFile)
        try:
            chosenTaxon = trainingFolderPAth

            if os.path.isfile(chosenTaxon):
                print ("will use this training file : " + chosenTaxon)
            else:
                print ("training file don't exist "+chosenTaxon)
                return "retry"
        except:
            print ("The training file you provided doesn't exist:")
            print (chosenTaxon)
            return "retry"
    else:
        chosenTaxon=False

    scripts_path = os.path.dirname(os.path.realpath(__file__))

    print ("Will use this number of cpus: " + str(cpuToUse))
    print ("Checking all programs are installed")

    print ("Checking Prodigal installed... " + str(which('prodigal')))

    starttime = "\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y")
    print (starttime)

    listOfGenomes = []

    fp = open(genomeFiles, 'r')

    for genomeFile in fp:
        genomeFile = genomeFile.rstrip('\n')
        genomeFile = genomeFile.rstrip('\r')
        listOfGenomes.append(genomeFile)

    fp.close()
    listOfGenomes.sort(key=lambda y: y.lower())

    # check if remnant files from previous run exist, prompt user if exists to know if it's his run and want to continue or start a new one

    basepath = os.path.join((os.path.dirname(outputFile)), "temp")
    if not os.path.exists(basepath):
        os.makedirs(basepath)

    # ------------------------------------------------- #
    #           RUN PRODIGAL OVER ALL GENOMES           #
    # ------------------------------------------------- #

    if inputCDS==True:
        CreateSchema.main(listOfGenomes[0], min_length, cpuToUse, False, outputFile, BlastpPath, bsr, verbose)
        shutil.rmtree(basepath)
        return True

    elif inputCDS==False:

        print ("\nStarting Prodigal at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

        # Prodigal run on the genomes, one genome per core using n-2 cores (n number of cores)
        print ("chosen taxon :" + str(chosenTaxon))
        pool = multiprocessing.Pool(cpuToUse)
        for genome in listOfGenomes:
            pool.apply_async(runProdigal.main,( str(genome), basepath, str(chosenTaxon)))

        pool.close()
        pool.join()

        print("\nChecking all prodigal processes created the necessary files...")

        listOfORFCreated = []
        for orffile in os.listdir(basepath):
            if orffile.endswith("_ORF.txt"):
                listOfORFCreated.append(orffile)

        if len(listOfGenomes) > len(listOfORFCreated):
            message = "Missing some files from prodigal. " + str(
                (len(listOfGenomes)) - (len(listOfORFCreated))) + " missing files out of " + str(len(listOfGenomes))
            shutil.rmtree(basepath)
            raise ValueError(message)
        else:
            print("All prodigal files necessary were created\n")

        print ("Finishing Prodigal at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    createSchemaPath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'CreateSchema.py')


    # ---CDS to protein---#

    # translate the genome CDSs, load them into dictionaries and fasta files to be used further ahead
    # listpairs=[]

    pairID = 0
    # while len(processed)<len(toprocess):
    with open("proteinID_Genome.tsv", 'w') as f:
        f.write("Genome\tcontig\tStart\tStop\tprotID")
    while len(listOfGenomes) > 0:

        pair = []
        dictPairs = {}

        for genomeFile in listOfGenomes:
            # toprocess.append(listOfGenomes)
            if len(pair) < 2:
                pair.append(genomeFile)
            else:
                dictPairs[pairID] = pair
                pairID += 1
                pair = []
                pair.append(genomeFile)

        # if total unpair, keep the remainig
        listOfGenomes = []

        if len(pair) == 2:
            dictPairs[pairID] = pair
            pairID += 1

        elif len(pair) > 0:
            listOfGenomes.append(pair[0])

        numberOfPairs = len(dictPairs.items())
        extraCpu = 0
        if numberOfPairs >= cpuToUse:
            pool = multiprocessing.Pool(cpuToUse)
        else:
            pool = multiprocessing.Pool(numberOfPairs)
            extraCpu = cpuToUse - numberOfPairs

        # print dictPairs
        for item in dictPairs.items():
            k = item[0]
            v = item[1]

            newgGenome = "protogenome" + str(k)
            pathFornewgGenome = os.path.join(basepath, newgGenome, newgGenome + ".fasta")
            listOfGenomes.append(pathFornewgGenome)
            extraCpuPerProcess = extraCpu / numberOfPairs
            print( "running analysis for pair : " + str(v[0]) + " " + str(v[1]))
            pool.apply_async(checkGeneStrings,
                             args=[v[0], v[1], newgGenome, basepath, int(extraCpuPerProcess + 1), BlastpPath,
                                   createSchemaPath,verbose,bsr])

        pool.close()
        pool.join()
        verboseprint( "finished running pair analysis")

        if len(listOfGenomes) == 1:

            verboseprint( "___________________\nFinal step : creating the schema")
            lastFile = listOfGenomes.pop()
            CreateSchema.main(lastFile,min_length,cpuToUse,False,outputFile,BlastpPath,bsr,verbose)
            #~ proc = subprocess.Popen(
            #~ [createSchemaPath, '-i', lastFile, '-l', str(min_length), '--cpu', str(cpuToUse), "-b", BlastpPath, "-o",
            #~ outputFile, "--bsr", str(bsr)])
            #~ p_status = proc.wait()
            verboseprint( "Schema Created sucessfully")

    shutil.rmtree(basepath)

    print (starttime)
    print ("Finished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import csv
from numpy import array
import argparse


def main():

    parser = argparse.ArgumentParser(description="Check if locus are being represented more than once")
    parser.add_argument('-i', nargs='?', type=str, help='raw file with allele call', required=True)


    args = parser.parse_args()
    contigsfile = args.i


    with open(contigsfile) as f:
        reader = csv.reader(f, delimiter="\t")
        d = list(reader)

    d2 = array(d)

    genelist= d2[:1,:]
    genelist=genelist.tolist()[0]
    genomeslist= d2[:,:1]
    genomeslist=genomeslist.tolist()
    genomeslist2=[]

    i=1
    pontuationDict={}


    while i<len(genomeslist):

        genomeSchema2=d2[i]
        genomeSchema=genomeSchema2.tolist()
        genomeSchema.pop(0)
        genomeSchema= "\t".join(genomeSchema)

        numberLOT=genomeSchema.count("LOT")
        numberLNF=genomeSchema.count("LNF")
        numberNIPH=genomeSchema.count("NIPH")
        numberASM=genomeSchema.count("ASM")
        numberALM=genomeSchema.count("ALM")

        numberMissingData=numberLOT+numberLNF+numberNIPH+numberASM+numberALM+0
        #if numberMissingData>0:
        pontuationDict[(genomeslist[i])[0]]=numberMissingData
        genomeslist2.append((genomeslist[i])[0])
        i+=1

    for k,v in pontuationDict.items():
        print(k,v)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
import argparse
from Bio import SeqIO
from Bio.Alphabet import generic_dna


def main():

    parser = argparse.ArgumentParser(description="This program removes alleles from a set of gene files, given a list of names of the genomes to be removed")
    parser.add_argument('-i', nargs='?', type=str, help='List of genes files (list of fasta files)', required=True)
    parser.add_argument('-g', nargs='?', type=str, help='List of genomes files which to remove', required=True)


    args = parser.parse_args()
    genes = args.i
    genomes = args.g

    gene_fp = open( genes, 'r')
    genomes_fp = open( genomes, 'r')
    listGenomesRemove=[]
    for genomes in genomes_fp:
        listGenomesRemove.append(genomes.rstrip('\n'))



    for gene in gene_fp:
        listSeqTokeep=[]
        gene = gene.rstrip('\n')
        print(gene)

        #per gene remove the alleles that belong to the genomes to be removed
        firstallele=False
        changed=False
        fasta2Write=''
        for allele in SeqIO.parse(gene, "fasta", generic_dna):


            alleleGenomeName=((str(allele.name)).split("_"))[-1]

            if not alleleGenomeName in listGenomesRemove:
                fasta2Write+=">"+allele.name+"\n"+allele.seq+"\n"
            else:
                changed=True
                print("removed: "+ str(allele.name))

            alleleGenomeName=((str(allele.name)).split("_"))[-1]

            #if to keep only 1st allele

            """if not firstallele:
                listSeqTokeep.append(allele)
                firstallele=True			
            else:
                print "removed: "+ str(allele.name)
            
            #if to remove the 1st allele
            try:
                int(allele.name)
                print "removed: "+ str(allele.name)
            except:
                listSeqTokeep.append(allele)"""
        
        if changed==True:
            with open(gene,'w') as f:
                f.write(fasta2Write)
        



if __name__ == "__main__":
    main()



#!/usr/bin/env python3
from Bio.Seq import Seq
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import argparse
import os.path
from Bio.Seq import Seq
from Bio.Blast.Applications import NcbiblastpCommandline
import collections
import shutil
try:
	from createschema import init_schema_4_bbaca,CommonFastaFunctions
except ImportError:
	from CHEWBBACA.createschema import init_schema_4_bbaca,CommonFastaFunctions
import time


def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True

    return "Not found"


def reverseComplement(strDNA):
    basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    strDNArevC = ''
    for l in strDNA:
        strDNArevC += basecomplement[l]

    return strDNArevC[::-1]


def translateSeq(DNASeq, genename):
    seq = DNASeq
    tableid = 11
    inverted = False
    
    #look for ambiguous base
    try:
        reverseComplement(seq)
    except:
        protseq = ""
    
    try:
        myseq = Seq(seq)
        protseq = Seq.translate(myseq, table=tableid, cds=True)
    except:
        try:
            seq = reverseComplement(seq)
            myseq = Seq(seq)
            protseq = Seq.translate(myseq, table=tableid, cds=True)
            inverted = True
        except:
            try:
                seq = seq[::-1]
                myseq = Seq(seq)
                protseq = Seq.translate(myseq, table=tableid, cds=True)
                inverted = True
            except:
                try:
                    seq = seq[::-1]
                    seq = reverseComplement(seq)
                    myseq = Seq(seq)
                    protseq = Seq.translate(myseq, table=tableid, cds=True)
                    inverted = False
                except Exception as e:
                    protseq = ""
    return protseq, seq, inverted


def main(genes,sizethresh,cpuToUse,proteinFIlePath,outputFIlePath,BlastpPath,bsr,verbose):
    #~ parser = argparse.ArgumentParser(
        #~ description="Given an ffn file, recovers the genes that are not paralogs and have a size bigger than the g parameter provided")
    #~ parser.add_argument('-i', nargs='?', type=str, help='ffn file', required=True)
    #~ parser.add_argument('-l', nargs='?', type=int, help='int minimum length', required=True)
    #~ parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus, if over the maximum uses maximum -2",
                        #~ required=False)
    #~ parser.add_argument('-p', nargs='?', type=str, help="file with protein", required=False, default=False)
    #~ parser.add_argument('-o', nargs='?', type=str, help="output filename", required=False, default=False)
    #~ parser.add_argument('-b', nargs='?', type=str, help="BLAST full path", required=False, default='blastp')
    #~ parser.add_argument('--bsr', nargs='?', type=float, help="minimum BSR similarity", required=False, default=0.6)
    #~ parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
                        #~ default=False)
#~ 
    #~ args = parser.parse_args()
    #~ genes = args.i
    #~ sizethresh = args.l
    #~ cpuToUse = args.cpu
    #~ proteinFIlePath = args.p
    #~ outputFIlePath = args.o
    #~ BlastpPath = args.b
    #~ bsr = args.bsr
    #~ verbose = args.verbose

    if verbose:
        def verboseprint(*args):

            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None
    
    starttime = "\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y")
    verboseprint ("\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    verboseprint ("Checking Blast installed... " + str(which(BlastpPath)))

    # translate to protein and create new file
    abspath = os.path.abspath(genes)
    filename = os.path.basename(genes)
    abspath = abspath.replace(filename, '')
    proteinfile = os.path.join(abspath, 'proteins.fasta')

    geneDict = {}
    protDict = {}
    orderedprotDict = collections.OrderedDict()
    alreadyIn = []
    totalgenes = 0
    repeatedgenes = 0
    smallgenes = 0
    nottranslatable = 0

    verboseprint ("Checking translatability of the loci:\n")

    if not proteinFIlePath:
        # print "not passing steps"
        with open(proteinfile, "w") as f:
            #g_fp = HTSeq.FastaReader(genes)

            for gene in SeqIO.parse(genes, "fasta", generic_dna):
                dnaseq = str(gene.seq.upper())
                protseq, seq, y = translateSeq(dnaseq, gene.id)
                totalgenes += 1
                if len(protseq) > 1:

                    if str(protseq) in alreadyIn:
                        repeatedgenes += 1

                    elif len(str(seq)) < sizethresh:
                        smallgenes += 1

                    else:
                        alreadyIn.append(str(protseq))
                        protname = ">" + str(gene.id) + "\n"

                        f.write(protname + str(protseq) + "\n")
                        protDict[protname] = str(protseq)
                        geneDict[str(gene.name)] = dnaseq
                else:
                    nottranslatable += 1
                    continue

            verboseprint (str(nottranslatable) + " not translatable out of " + str(totalgenes))

            verboseprint ( "\nChecking if repeated protein sequences:\n")

            orderedprotList = []
            orderedprotList = sorted(protDict.items(), key=lambda x: len(x[1]), reverse=True)

            i = 0
            while i < len(orderedprotList):
                elem = orderedprotList[i]
                orderedprotDict[elem[0]] = elem[1]
                i += 1

        verboseprint (str(repeatedgenes) + " repeated loci out of " + str(totalgenes))
        verboseprint (str(smallgenes) + " loci out of " + str(totalgenes) + " smaller than " + str(sizethresh) + "bp")
        verboseprint ("\nprotein file created\n")

        # first step -  remove genes contained in other genes or 100% equal genes

        # list of results - the output of the function
        resultsList = []

        auxDict = {}
        #g_fp = HTSeq.FastaReader(proteinfile)
        g = 0
        j = 0

        verboseprint ( "Checking if protein sequences are contained in others...")

        # for each gene from all the annotated genes - starting with an empty dictionary, only add a new gene if the "to be added gene" is not contained or equal to a gene already added to the dictionary
        auxprot = []

        for elem in orderedprotDict.items():

            contained = False

            prot = str(elem[1])
            if any(prot in x for x in auxprot):
                g += 1
                contained = True

            else:
                auxDict[elem[1]] = elem[0]
                auxprot.append(str(elem[1]))

            j += 1
        verboseprint ( str(g)+" loci are contained in other genes\n")

        # overwrite the original file, obtaining a new file with unique genes

        with open(proteinfile, "w") as f:
            allsequences = ''
            for k, v in auxDict.items():
                allsequences += v + k + "\n"
            f.write(allsequences)

    else:
        # print "passed steps"

        proteinfile = proteinFIlePath
        totalgenes = 0
        smallgenes = 0
        #g_fp = HTSeq.FastaReader(genes)
        proteinfile = proteinFIlePath
        for gene in SeqIO.parse(genes, "fasta", generic_dna):
        #for gene in g_fp:
            dnaseq = str(gene.seq.upper())

            protname = ">" + str(gene.id) + "\n"
            # protDict[protname] = str(protseq)
            geneDict[str(gene.name)] = dnaseq

    verboseprint ( "Starting Blast")
    # print "Blasting the total of "+ str(len(auxDict.keys())) + " loci"

    geneFile = os.path.abspath(proteinfile)
    Gene_Blast_DB_name = CommonFastaFunctions.Create_Blastdb(geneFile, 1, True)

    geneF = os.path.splitext(geneFile)[0]
    blast_out_file = geneF + '.xml'
    # ------------------------------ RUNNING BLAST ------------------------------ #
    if cpuToUse:
        cline = NcbiblastpCommandline(cmd=BlastpPath, query=geneFile, db=Gene_Blast_DB_name, evalue=0.001,
                                      out=blast_out_file, outfmt=5, num_threads=int(cpuToUse))
    else:
        cline = NcbiblastpCommandline(cmd=BlastpPath, query=geneFile, db=Gene_Blast_DB_name, evalue=0.001,
                                      out=blast_out_file, outfmt=5, num_threads=1)
    blast_records = CommonFastaFunctions.runBlastParser(cline, blast_out_file)
    verboseprint ( "Finished blast")

    toRemove = []
    genesToKeep = []
    log = ["removed\tcause\texplanation"]
    for blast_record in blast_records:

        allelename = blast_record.query
        allelename = allelename.split(" ")
        allelename = allelename[0]
        alleleLength = len(geneDict[allelename])

        try:

            # if gene A is not on the toRemove list yet, add to genesToKeep list

            if str(blast_record.query) not in toRemove:
                genesToKeep.append(blast_record.query)

                i = 0
                # if first alignement is not against self, gene B is bigger than gene A and very simillar - remove gene A from genesToKeep and add gene B instead
                if not str(blast_record.query) == str((blast_record.alignments[0]).hit_def):
                    genesToKeep.remove(str(blast_record.query))
                    toRemove.append(str(blast_record.query))
                    log.append(str(blast_record.query) + "\t" + str(
                        (blast_record.alignments[0]).hit_def) + "\t" + "2 is first best match")

                    # if gene B is not on the toRemove list, add to genesToKeep list
                    if str((blast_record.alignments[0]).hit_def) not in toRemove:
                        genesToKeep.append(str((blast_record.alignments[0]).hit_def))

                    raise Exception

                selfblastscore = (((blast_record.alignments[0]).hsps)[0]).score

                while i < len(blast_record.alignments):
                    align = blast_record.alignments[i]

                    match = (align.hsps)[0]
                    scoreRatio = float(match.score) / float(selfblastscore)

                    alleleLength2 = len(geneDict[str(align.hit_def)])

                    # if good match and gene B not in toremove list
                    if (scoreRatio > bsr and not str(align.hit_def) == str(blast_record.query) and str(
                            align.hit_def) not in toRemove):

                        # if gene B is bigger than gene A, keep bigger gene B
                        if alleleLength2 > alleleLength:
                            genesToKeep.append(str(align.hit_def))
                            genesToKeep.remove(str(blast_record.query))
                            toRemove.append(str(blast_record.query))
                            log.append(str(blast_record.query) + "\t" + str(
                                align.hit_def) + "\t" + "2 is bigger and bsr >" + str(bsr))

                            raise Exception
                        # else add gene B to toremove list
                        elif str(align.hit_def) in genesToKeep:
                            genesToKeep.remove(str(align.hit_def))
                            toRemove.append(str(align.hit_def))
                            log.append(str(align.hit_def) + "\t" + str(
                                blast_record.query) + "\t" + "2 is bigger and bsr >" + str(bsr))

                    i += 1

            # else gene A is on toRemove list, add all similar genes (not in genesToKeep) list to the toRemove list
            else:

                i = 0
                selfblastscore = 0
                for align in blast_record.alignments:
                    if not (str(align.hit_def) == str(blast_record.query)):
                        selfblastscore = ((align.hsps)[0]).score
                        # print "gene "+str(align.hit_def)+" is larger than gene "+str(blast_record.query)
                        raise Exception

                while i < len(blast_record.alignments):
                    align = blast_record.alignments[i]
                    match = (align.hsps)[0]
                    scoreRatio = float(match.score) / float(selfblastscore)

                    if align.hit_def not in genesToKeep and not str(align.hit_def) == str(
                            blast_record.query) and scoreRatio > bsr:
                        toRemove.append(align.hit_def)
                        log.append(str(align.hit_def) + "\t" + str(
                            blast_record.query) + "\t" + "2 was on the removed list and bsr >" + str(bsr))

                    else:
                        pass

                    i += 1


        except Exception as e:
            # print e
            pass

    genesToKeep = list(set(genesToKeep))
    toRemove = list(set(toRemove))
    s = set(toRemove)
    notcommonToKeep = [x for x in genesToKeep if x not in s]

    pathfiles = os.path.dirname(geneFile)
    pathfiles = pathfiles + "/"
    listfiles = []

    #g_fp = HTSeq.FastaReader(genes)
    removedparalogs = 0
    removedsize = 0
    totalgenes = 0
    rest = 0
    concatenatedFile = ''
    schema_folder_path = os.path.join(pathfiles, 'schema_seed')

    if not os.path.exists(schema_folder_path) and not proteinFIlePath and not outputFIlePath:
        os.makedirs(schema_folder_path)
    elif not proteinFIlePath and outputFIlePath:
        os.makedirs(outputFIlePath)

    for contig in SeqIO.parse(genes, "fasta", generic_dna):
        totalgenes += 1
        #name = contig.name + " " + contig.descr
        name2 = contig.id

        # print name2
        if name2 not in toRemove and name2 in genesToKeep:
            if int(len(contig.seq)) > sizethresh:
                namefile = contig.name
                namefile = namefile.replace("|", "_")
                namefile = namefile.replace("_", "-")
                namefile = namefile.replace("(", "")
                namefile = namefile.replace(")", "")
                namefile = namefile.replace("'", "")
                namefile = namefile.replace("\"", "")
                namefile = namefile.replace(":", "")

                if not proteinFIlePath and not outputFIlePath:
                    newFile = os.path.join(schema_folder_path, namefile + ".fasta")
                    listfiles.append(newFile)
                    with open(newFile, "w") as f:
                        f.write(">" + namefile + "_1\n" + str(contig.seq).upper() + "\n")
                elif not proteinFIlePath and outputFIlePath:
                    newFile = os.path.join(outputFIlePath, namefile + ".fasta")
                    listfiles.append(newFile)
                    with open(newFile, "w") as f:
                        f.write(">" + namefile + "_1\n" + str(contig.seq).upper() + "\n")
                else:
                    concatenatedFile += ">" + contig.id + " \n" + str(contig.seq.upper()) + "\n"

                rest += 1

            else:
                removedsize += 1
        else:

            removedparalogs += 1

    if proteinFIlePath and outputFIlePath:
        with open(outputFIlePath, "w") as f:
            f.write(concatenatedFile)
    elif not proteinFIlePath and outputFIlePath:
        init_schema_4_bbaca.get_Short(listfiles)
        verboseprint ( "\nRemoved "+str(removedparalogs)+" with a high similarity (BSR>"+str(bsr)+")")
        print ("Total of "+str(rest)+" loci that constitute the schema")
        os.remove(proteinfile)

    # create short folder
    else:
        # ~ with open("schemacreation.log", "wb") as f:
        # ~ for elem in log:
        # ~ f.write(str(elem)+"\n")
        init_schema_4_bbaca.get_Short(listfiles)
        verboseprint ( "\nRemoved "+str(removedparalogs)+" with a high similarity (BSR>"+str(bsr)+")")
        print ("Total of "+str(rest)+" loci that constitute the schema")
        os.remove(proteinfile)

    shutil.rmtree(os.path.join(pathfiles, 'blastdbs'))

    os.remove(blast_out_file)

    verboseprint (starttime)
    verboseprint ("Finished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import sys
import os
import subprocess
import pickle


def main(input_file,tempPath,choosenTaxon):

    contigsFasta = input_file

    basepath = tempPath

    # ------------ #
    # RUN PRODIGAL #
    # ------------ #
    # prodigal_path='prodigal'

    if choosenTaxon == "False":

        proc = subprocess.Popen(
            ['prodigal', '-i', contigsFasta, '-c', '-m', '-g', '11', '-p', 'single', '-f', 'sco', '-q'],
            stdout=subprocess.PIPE)
    else:
        proc = subprocess.Popen(
            ['prodigal', '-i', contigsFasta, '-c', '-m', '-g', '11', '-p', 'single', '-f', 'sco', '-q', '-t',
             choosenTaxon], stdout=subprocess.PIPE)

    cdsDict = {}
    tempList = []
    line = ' '
    while line != '':

        # when it finds a contig tag
        if "seqhdr" in line:
            # add contig to cdsDict and start new entry

            if len(tempList) > 0:

                # --- brute force parsing of the contig tag - better solution is advisable --- #

                i = 0
                for l in contigTag:
                    if l == ' ':
                        break
                    i += 1
                contigTag = contigTag[:i]

                cdsDict[contigTag.replace("\r", "")] = tempList
                tempList = []

            contigTag = line.split('"')[-2]

        # when it finds a line with cds indexes
        elif line[0] == '>':

            # parsing
            cdsL = line.split('_')

            # --- each element of this list is a pair of indices - the start and the end of a CDS --- #

            tempList.append([int(cdsL[1]) - 1, int(
                cdsL[2])])  # start index correction needed because prodigal indexes start in 1 instead of 0

        # reads the stdout from 'prodigal'
        line = proc.stdout.readline().decode("utf-8")

    # ADD LAST
    if len(tempList) > 0:

        # --- brute force parsing of the contig tag - better solution is advisable --- #

        i = 0
        for l in contigTag:
            if l == ' ':
                break
            i += 1
        contigTag = contigTag[:i]

        cdsDict[contigTag.replace("\r", "")] = tempList

    filepath = os.path.join(basepath, str(os.path.basename(contigsFasta)) + "_ORF.txt")
    with open(filepath, 'wb') as f:
        var = cdsDict
        pickle.dump(var, f)

    print("done prodigal run on:" + str(os.path.basename(contigsFasta)))

    return True


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from Bio.Seq import Seq
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import os
import argparse
from SPARQLWrapper import SPARQLWrapper, JSON
import csv
from collections import defaultdict
import multiprocessing
virtuoso_server=SPARQLWrapper('http://sparql.uniprot.org/sparql')

class Result():
    def __init__(self):
        self.val = []

    def update_result(self, val):
        lala=val
        self.val.append(lala)

    def get_result(self):
        return self.val

def translateSeq(DNASeq, verbose):
    if verbose:
        def verboseprint(*args):
            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function

    seq = DNASeq
    tableid = 11
    inverted = False

    myseq = Seq(seq)
    protseq = Seq.translate(myseq, table=tableid, cds=True)


    return str(protseq)

def check_if_list_or_folder(folder_or_list):
    list_files = []
    # check if given a list of genomes paths or a folder to create schema
    try:
        f = open(folder_or_list, 'r')
        f.close()
        list_files = folder_or_list
    except IOError:

        for gene in os.listdir(folder_or_list):
            try:
                genepath = os.path.join(folder_or_list, gene)
                for allele in SeqIO.parse(genepath, "fasta", generic_dna):
                    break
                list_files.append(os.path.abspath(genepath))
            except Exception as e:
                print (e)
                pass

    return list_files


def get_data(sparql_query):
    virtuoso_server.setQuery(sparql_query)
    virtuoso_server.setReturnFormat(JSON)
    virtuoso_server.setTimeout(10)
    try:
        result = virtuoso_server.query().convert()
    except:
        print ("A request to uniprot timed out, trying new request")
        time.sleep(5)
        result = virtuoso_server.query().convert()
    return result

def get_protein_info(proteinSequence):
    proteinSequence=proteinSequence.replace("*","")

    name=''
    url=''
    prevName=''
    prevUrl=''

    query='PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>  PREFIX up: <http://purl.uniprot.org/core/> select ?seq ?fname ?fname2 ?fname3  where {{?b a up:Simple_Sequence; rdf:value "'+proteinSequence+'". ?seq up:sequence ?b. OPTIONAL{?seq up:submittedName ?sname. ?sname up:fullName ?fname2} OPTIONAL{?seq up:recommendedName ?rname.?rname up:fullName ?fname} }UNION{?seq a up:Sequence; rdf:value "'+proteinSequence+'"; rdfs:label ?fname3. }}'
    result = get_data(query)
    #~ print (query)

    try:
        result["results"]["bindings"][0]
        aux=result["results"]["bindings"]
        for elem in aux:
            if 'fname' in elem.keys():
                name=str(elem['fname']['value'])
                url=str(elem['seq']['value'])
            elif 'fname2' in elem.keys():
                name=str(elem['fname2']['value'])
                url=str(elem['seq']['value'])
            elif 'fname3' in elem.keys():
                name=str(elem['fname3']['value'])
                url=str(elem['seq']['value'])


            if not "Uncharacterized protein" in name:
                break

            if prevName=='' and (not "Uncharacterized protein" in name or not "hypothetical" in name or not "DUF" in name):
                prevName=name
                prevUrl=url
            else:
                name=prevName
                url=prevUrl

            #~ print (name)
            #~ print (url)

    except Exception as e:
        return False




    return str(name),str(url)

def proc_gene(gene,auxBar):

    #~ print gene
    name=''
    url=''
    prevName=''
    prevUrl=''
    for allele in SeqIO.parse(gene, "fasta", generic_dna):
        params = {}
        sequence=str(allele.seq)
        try:
            proteinSequence=translateSeq(sequence,False)
        except:
            continue
        try:
            name,url=get_protein_info(proteinSequence)
            if "Uncharacterized protein" in name or "hypothetical" in name or "DUF" in name :
                if not prevName=="":
                    name=prevName
                    url=prevUrl
                #~ print("trying next allele")
                continue
            else:
                prevName=name
                prevUrl=url
                #~ print (name)
                #~ print (url)
                break
        except Exception as e:
            #~ print (e)
            #~ print("trying next allele")
            continue

    if gene in auxBar:
        auxlen=len(auxBar)
        index=auxBar.index(gene)
        print ( "["+"="*index+">"+" "*(auxlen-index)+"] Querying "+str(int((float(index)/auxlen)*100))+"%")

    return [gene, name, url]

def main(geneFiles,proteinid2genome,cpu2use):
    #~ parser = argparse.ArgumentParser(
    #~ description="This program get names for a schema")
    #~ parser.add_argument('-i', nargs='?', type=str, help='path to folder containg the schema fasta files ( alternative a list of fasta files)', required=True)
    #~ parser.add_argument('-t', nargs='?', type=str, help='path to tsv file)', required=True)
    #~ parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu', required=False, default=1)
    #~
    #~ args = parser.parse_args()
    #~
    #~ geneFiles = args.i
    #~ proteinid2genome = args.t
    #~ cpu2use = args.cpu

    geneFiles = check_if_list_or_folder(geneFiles)
    if isinstance(geneFiles, list):
        with open("listGenes.txt", "w") as f:
            for genome in geneFiles:
                f.write(genome + "\n")
        geneFiles = "listGenes.txt"

    listGenes=[]
    gene_fp = open( geneFiles, 'r')
    for gene in gene_fp:
        gene = gene.rstrip('\n')
        listGenes.append(gene)
    gene_fp.close()

    try:
        os.remove("listGenes.txt")
    except:
        pass

    dictaux= defaultdict(list)

    with open(proteinid2genome) as csvfile:
        reader = csv.reader(csvfile, delimiter='\t')
        headers = next(reader)
        for row in reader:
            for elem in row:
                dictaux[row[0]+row[-1]].append(str(elem))



    print ("Processing the fastas")
    got=0
    notgot=[]
    uncharacterized=[]
    selected_prots=[]
    counter=0


    #test down bar
    auxBar=[]
    #~ orderedkeys=sorted(newDict.keys())
    step=(int((len(listGenes))/10))+1
    counter=0
    while counter < len(listGenes):
        auxBar.append(listGenes[counter])
        counter+=step

    pool = multiprocessing.Pool(cpu2use)
    result = Result()
    for gene in listGenes:
        p=pool.apply_async(proc_gene,args=[gene,auxBar],callback=result.update_result)

    pool.close()
    pool.join()
    listResults=result.get_result()


    for result in listResults:
        #~ lala=process_locus(gene,path2schema,newDict,auxBar)
        gene=result[0]
        name=result[1]
        url=result[2]

        print ("final name: "+name)
        if "Uncharacterized protein" in name or "hypothetical" in name:
            uncharacterized.append(gene)
            got+=1

        elif name=="":
            notgot.append(gene)
        else:
            got+=1

        aux=gene.split("-protein")
        protid=aux[-1].replace(".fasta","")
        aux2=aux[0].split("/")[-1].replace("-","_")
        dictaux[aux2+protid].append(str(name))
        dictaux[aux2+protid].append(str(url))
        dictaux[aux2+protid].insert(0,os.path.basename(gene))
        selected_prots.append(aux2+protid)


    newProfileStr="Fasta\t"+('\t'.join(map(str, headers)))+"\tname\turl\n"
    for key in set(sorted(selected_prots,key=str)):
        newProfileStr += ('\t'.join(map(str, dictaux[key])))+"\n"

    print ("Found : " +str(got)+ ", "+str(len(uncharacterized))+" of them uncharacterized/hypothetical. Nothing found on :"+str(len(notgot)))
    with open("new_protids.tsv", "w") as f:
        f.write(newProfileStr)

    print ("Done")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
import argparse
import multiprocessing
from Bio import SeqIO
from Bio.Alphabet import generic_dna
from Bio.Seq import Seq


def reverseComplement(strDNA):

	basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
	strDNArevC = ''
	for l in strDNA:

		strDNArevC += basecomplement[l]

	return strDNArevC[::-1]

def translateSeq(DNASeq,transTable):
	seq=DNASeq
	tableid=transTable
	reversedSeq=False
	try:
		myseq= Seq(seq)
		protseq=Seq.translate(myseq, table=tableid,cds=True)
	except:
		reversedSeq=True
		try:
			seq=reverseComplement(seq)
			myseq= Seq(seq)
			protseq=Seq.translate(myseq, table=tableid,cds=True)

		except:
			try:
				seq=seq[::-1]
				myseq= Seq(seq)
				protseq=Seq.translate(myseq, table=tableid,cds=True)
			except:
				reversedSeq=False
				try:
					seq=seq[::-1]
					seq=reverseComplement(seq)
					myseq= Seq(seq)
					protseq=Seq.translate(myseq, table=tableid,cds=True)
				except Exception as e:
					print (e)
					raise ValueError(e)
	return protseq,seq,reversedSeq

def curate(geneFile):

	gene2write=''
	#gene_fp2 = HTSeq.FastaReader(geneFile)
	for allele in SeqIO.parse(geneFile, "fasta", generic_dna):
		sequence = str(allele.seq.upper())
		name = allele.name
		#per gene remove the alleles that are not CDS
		#for allele in gene_fp2:

		# if allele is not multiple of 3 it's useless to try to translate
		if (len(sequence) % 3 != 0):

			pass
		else:
			try:
				protseq,seq,reversedSeq=translateSeq(sequence, 11)
				gene2write+=">"+name+"\n"+sequence+"\n"

			except Exception as err:
				print(err)


	with open(geneFile, "wb") as f:
		f.write(gene2write)
	return True

def main():

	parser = argparse.ArgumentParser(description="This program removes alleles from a set of gene files, given a list of names of the genomes to be removed")
	parser.add_argument('-i', nargs='?', type=str, help='List of genes files (list of fasta files)', required=True)
	parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus", required=True)

	args = parser.parse_args()
	genes = args.i
	cpuToUse=args.cpu

	gene_fp = open( genes, 'r')

	pool = multiprocessing.Pool(cpuToUse)
	for gene in gene_fp:
		print (gene)
		gene = gene.rstrip('\n')

		pool.apply_async(curate,args=[str(gene)])

	pool.close()
	pool.join()




if __name__ == "__main__":
	main()



#!/usr/bin/env python3
from io import StringIO
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import sys
from Bio.Seq import Seq
from Bio.Blast.Applications import NcbiblastpCommandline
from collections import Counter
import os
from Bio.Blast import NCBIXML
try:
	from allelecall import CommonFastaFunctions
except ImportError:
	from CHEWBBACA.allelecall import CommonFastaFunctions
import time
import pickle
import shutil
import warnings
from Bio import BiopythonWarning
warnings.simplefilter('ignore', BiopythonWarning)


def getBlastScoreRatios(genefile, basepath, doAll, verbose, blastPath):
    if verbose:
        def verboseprint(*args):
            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function

    #gene_fp = HTSeq.FastaReader(genefile)
    allelescores = []
    alleleProt = ''
    alleleAllProt = ''
    alleleList = []
    alleleI = 0
    alleleIlist = []
    listAllelesNames = []
    # calculate bsr for each allele
    for allele in SeqIO.parse(genefile, "fasta", generic_dna):

        # usually first allele name is just >1 and after that it has >gene_id_genome
        aux = allele.id.split("_")
        if len(aux) < 2:
            alleleI = int(aux[0])
        else:
            alleleI = int(aux[-1])

        # try to translate the allele
        alleleIlist.append(alleleI)
        alleleList.append(str(allele.seq.upper()))
        listAllelesNames.append(allele.id)
        translatedSequence, x = translateSeq(str(allele.seq.upper()))

        if translatedSequence == '':
            print ("cannot translate allele on bsr calculation")
            pass

        # calculate BSR for the allele
        else:
            alleleProt = ">" + str(alleleI) + "\n" + str(translatedSequence + "\n")
            alleleAllProt += ">" + str(alleleI) + "\n" + str(translatedSequence + "\n")
            proteinfastaPath = os.path.join(basepath, str(os.path.basename(genefile) + '_protein2.fasta'))

            # new db for each allele to blast it against himself

            Gene_Blast_DB_name = CommonFastaFunctions.Create_Blastdb_no_fasta(proteinfastaPath, 1, True,alleleProt)

            # if bsr hasn't been calculated, do the BLAST
            if doAll:

                #~ blast_out_file = os.path.join(basepath, 'blastdbs/temp.xml')
                verboseprint("Starting Blast alleles at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

                # --- get BLAST score ratio --- #
                #~ cline = NcbiblastpCommandline(cmd=blastPath, query=proteinfastaPath, db=Gene_Blast_DB_name,
                                              #~ evalue=0.001, out=blast_out_file, outfmt=5, num_threads=1)
                                              
                cline = NcbiblastpCommandline(cmd=blastPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5,num_threads=1)
                out, err = cline(stdin=alleleProt)
                psiblast_xml = StringIO(out)
                blast_records = NCBIXML.parse(psiblast_xml)
                
                
                allelescore = 0

                #~ blast_records = CommonFastaFunctions.runBlastParser(cline, blast_out_file)

                verboseprint("Blasted alleles at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

                for blast_record in blast_records:

                    for alignment in blast_record.alignments:

                        for match in alignment.hsps:
                            allelescores.append(int(match.score))

                geneScorePickle = os.path.abspath(genefile) + '_bsr.txt'
                verboseprint("________")
                # ~ var=[alleleI,allelescores]
                var = dict(zip(alleleIlist, allelescores))
                with open(geneScorePickle, 'wb') as f:
                    pickle.dump(var, f)

            # bsr had already been calculated, load it to memory
            else:
                geneScorePickle = os.path.abspath(genefile) + '_bsr.txt'
                with open(geneScorePickle, 'rb') as f:
                    var = pickle.load(f)
                # ~ allelescores=var[1]

    proteinfastaPath = os.path.join(basepath, str(os.path.basename(genefile) + '_protein.fasta'))
    with open(proteinfastaPath, "w") as f:
        f.write(alleleAllProt)

    # returning all allele BSR scores and list of alleles for this gene
    return var, alleleList, listAllelesNames


def reDogetBlastScoreRatios(sequence, basepath, alleleI, allelescores2, newGene_Blast_DB_name, alleleList2, picklepath,
                            verbose, blastPath, listAllelesNames):
    if verbose:
        def verboseprint(*args):
            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function

    #gene_fp = HTSeq.FastaReader(genefile)

    alleleProt = ''

    #~ proteinfastaPath = genefile

    verboseprint("Starting Blast of new alleles to calculate BSR at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    #~ blast_out_file2 = os.path.join(basepath, 'blastdbs/temp.xml')

    cline = NcbiblastpCommandline(cmd=blastPath, db=newGene_Blast_DB_name, evalue=0.001, outfmt=5,num_threads=1)
    
    out, err = cline(stdin=sequence)
    
    allelescore = 0
    #~ blast_records = runBlastParser(cline, blast_out_file2)
    #~ rec = open(out)
    psiblast_xml = StringIO(out)
    blast_records = NCBIXML.parse(psiblast_xml)

    verboseprint("Blasted alleles at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    found = False
    matchscore = 0
    for blast_record in blast_records:

        for alignment in blast_record.alignments:

            for match in alignment.hsps:
                matchscore = int(match.score)

    
    allelescores2[alleleI] = matchscore
    with open(picklepath, 'wb') as f:
        pickle.dump(allelescores2, f)

    return allelescores2, alleleList2, listAllelesNames


def reverseComplement(strDNA):
    basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    strDNArevC = ''
    for l in strDNA:
        strDNArevC += basecomplement[l]

    return strDNArevC[::-1]


def translateSeq(DNASeq):
    seq = DNASeq
    reversedSeq = False
    tableid = 11
    try:
        myseq = Seq(seq)
        protseq = Seq.translate(myseq, table=tableid, cds=True)
    except:
        reversedSeq = True
        try:
            seq = reverseComplement(seq)
            myseq = Seq(seq)
            protseq = Seq.translate(myseq, table=tableid, cds=True)

        except:
            try:
                seq = seq[::-1]
                myseq = Seq(seq)
                protseq = Seq.translate(myseq, table=tableid, cds=True)
            except:
                reversedSeq = False
                try:
                    seq = seq[::-1]
                    seq = reverseComplement(seq)
                    myseq = Seq(seq)
                    protseq = Seq.translate(myseq, table=tableid, cds=True)
                except Exception as e:
                    print ("translation error")
                    print (e)
                    protseq = ""
    return protseq, seq


# ======================================================== #
#            Allele calling and classification             #
# ======================================================== #
def main(input_file,temppath,blastPath,verbose,bsrTresh,sizeTresh):
    #~ try:
        #~ input_file = sys.argv[1]
        #~ temppath = sys.argv[2]
        #~ blastPath = sys.argv[3]
        #~ verbose = sys.argv[4]
        #~ bsrTresh = sys.argv[5]

    if verbose == 'True':
        verbose = True
    else:
        verbose = False

    #~ except IndexError:
        #~ print ("Error starting the callAlleleles_protein3 script. usage: list_pickle_obj")

    bsrTresh = float(bsrTresh)
    sizeTresh = float(sizeTresh)

    argumentList = []
    with open(input_file, 'rb') as f:
        argumentList = pickle.load(f)

    if verbose:
        def verboseprint(*args):

            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None

    geneFile = argumentList[0]

    verboseprint("Using gene: " + str(geneFile))
    shortgeneFile = os.path.join(os.path.dirname(argumentList[0]), "short", os.path.basename(argumentList[0]))
    shortgeneFile = shortgeneFile.replace(".fasta", "_short.fasta")
    genomesList = argumentList[1]
    genesList = argumentList[2]

    newListgenes = []
    with open(genesList, 'r') as gene_fp:
        for gene in gene_fp:
            gene = gene.rstrip('\n')
            gene = gene.rstrip('\r')
            newListgenes.append(gene)

    statusbar = float(newListgenes.index(str(geneFile))) / len(newListgenes)
    locusnumber = (newListgenes.index(str(geneFile)))
    totalocusnumber = len(newListgenes)
    basepath = os.path.join(temppath, os.path.splitext(geneFile)[0])
    newDNAAlleles2Add2Fasta=''
    newDNAAlleles2Add2shortFasta=''
    proteinFastaString=''

    print (
    "\rProcessing " + os.path.basename(geneFile) + ". Start " + time.strftime("%H:%M:%S-%d/%m/%Y") + " Locus " + str(
        locusnumber) + " of " + str(totalocusnumber) + ". Done " + str(int(statusbar * 100)) + "%.", end="")

    if not os.path.exists(basepath):
        os.makedirs(basepath)

    #gene_fp = HTSeq.FastaReader(geneFile)

    fullAlleleList = []
    fullAlleleNameList = []
    alleleI = 0
    # get full list of alleles from main gene file and last allele number id
    for allele in SeqIO.parse(geneFile, "fasta", generic_dna):
        aux = allele.id.split("_")
        if len(aux) < 2:
            alleleI = int(aux[0])
        else:
            alleleI = int(aux[-1])
        fullAlleleList.append(str(allele.seq.upper()))
        fullAlleleNameList.append(allele.id)

    resultsList = []
    i = 0
    perfectMatchIdAllele = []
    perfectMatchIdAllele2 = []
    allelescores = []
    listShortAllelesNames = []

    verboseprint("Getting BSR at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    geneScorePickle = os.path.abspath(shortgeneFile) + '_bsr.txt'

    # check if bsr as arealdy been calculated and recalculate it if necessary

    if os.path.isfile(geneScorePickle):
        allelescores, alleleList, listShortAllelesNames = getBlastScoreRatios(shortgeneFile, basepath, False, verbose,
                                                                              blastPath)

    else:
        allelescores, alleleList, listShortAllelesNames = getBlastScoreRatios(shortgeneFile, basepath, True, verbose,
                                                                              blastPath)

    with open(os.path.join(basepath, str(os.path.basename(shortgeneFile) + '_protein.fasta')), 'r') as myfile:
        proteinFastaString = myfile.read()
        proteinFastaString+="\n"
    
    verboseprint("Finished BSR at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    verboseprint("starting allele call blast at: " + time.strftime("%H:%M:%S-%d/%m/%Y"))
    for genomeFile in genomesList:
        verboseprint(genomeFile)
        bestmatch = [0, 0, False, '',
                     0]  # score, score ratio, perfectmatch, key name of the DNA sequence string, allele ID
        currentGenomeDict = {}
        currentCDSDict = {}

        # load the CDS from the genome to a dictionary
        filepath = os.path.join(temppath, str(os.path.basename(genomeFile)) + "_ORF_Protein.txt")

        with open(filepath, 'rb') as f:
            currentCDSDict = pickle.load(f)

        try:
            intersection = set(fullAlleleList).intersection(currentCDSDict.values())
            intersection = list(intersection)

            if len(intersection) > 1:
                perfectMatchIdAllele.append('NIPHEM')
                perfectMatchIdAllele2.append('NIPHEM')
                verboseprint(os.path.basename(genomeFile) + " has " + str(
                    len(intersection)) + " multiple exact match : " + os.path.basename(
                    geneFile) + " MULTIPLE ALLELES as EXACT MATCH")
                raise ValueError("MULTIPLE ALLELES as EXACT MATCH")


            elif len(intersection) == 1:
                alleleStr = intersection[0]
                # it doenst return both keys with equal values
                # ~ elem=currentCDSDict.keys()[currentCDSDict.values().index(alleleStr)]

                elem = [key for key, value in currentCDSDict.items() if value == alleleStr]
                if len(elem) > 1:
                    perfectMatchIdAllele.append('NIPHEM')
                    perfectMatchIdAllele2.append('NIPHEM')
                    verboseprint(os.path.basename(genomeFile) + " has " + str(
                        len(intersection)) + " multiple exact match : " + os.path.basename(
                        geneFile) + " MULTIPLE ALLELES as EXACT MATCH")
                    raise ValueError("MULTIPLE ALLELES as EXACT MATCH")

                contigname = elem[0].split("&")
                matchLocation = contigname[2]
                # starting CDS base need to be +1
                matchLocation = matchLocation.split("-")
                matchLocation = [int(matchLocation[0]) + 1, int(matchLocation[1])]
                contigname = (contigname[0]).replace(">", "")
                alleleName = ''
                alleleMatchid = 0

                alleleName = fullAlleleNameList[fullAlleleList.index(alleleStr)]
                alleleMatchid = int((alleleName.split("_"))[-1])
                perfectMatchIdAllele.append(str(alleleMatchid))

                if matchLocation[0] > matchLocation[1]:
                    perfectMatchIdAllele2.append(
                        str(contigname) + "&" + str(matchLocation[0]) + "-" + str(matchLocation[1]) + "&" + "-")
                else:

                    perfectMatchIdAllele2.append(
                        str(contigname) + "&" + str(matchLocation[0]) + "-" + str(matchLocation[1]) + "&" + "+")

                # check if atributed allele is contained or contains
                try:
                    containedInfo = (alleleName.split("_"))[1]
                except:
                    containedInfo = ''
                if containedInfo == "CD":
                    resultsList.append([(os.path.basename(genomeFile)), str(alleleMatchid), containedInfo.rstrip()])
                elif containedInfo == "CS":
                    resultsList.append([(os.path.basename(genomeFile)), str(alleleMatchid), containedInfo.rstrip()])
                else:
                    pass

                raise ValueError("EQUAL")
        except Exception as e:
            # ~ exc_type, exc_obj, exc_tb = sys.exc_info()
            # ~ fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            # ~ print(exc_tb.tb_lineno)
            # ~ print e
            continue

        else:
            verboseprint("Blasting alleles on genome at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

            blast_out_file = os.path.join(basepath, "blastdbs/" + os.path.basename(geneFile) + '_List.xml')

            Gene_Blast_DB_name = os.path.join(temppath, str(os.path.basename(genomeFile)) + "/" + str(
                os.path.basename(genomeFile)) + "_db")

            #~ proteinfastaPath = os.path.join(basepath, str(os.path.basename(shortgeneFile) + '_protein.fasta'))

            # blast the genome CDS against the translated locus
            # cline = NcbiblastpCommandline(query=proteinfastaPath, db=Gene_Blast_DB_name, evalue=0.001, out=blast_out_file, outfmt=5,max_target_seqs=10,max_hsps_per_subject=10)
            # 2.2.28 up
            cline = NcbiblastpCommandline(cmd=blastPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5, max_target_seqs=10, max_hsps=10,num_threads=1)
    
            out, err = cline(stdin=proteinFastaString)
            
            #~ proteinFastaString
            #~ cline = NcbiblastpCommandline(cmd=blastPath, query=proteinfastaPath, db=Gene_Blast_DB_name, evalue=0.001, outfmt=5, max_target_seqs=10, max_hsps=10,num_threads=1)
            #~ out, err = cline()
            psiblast_xml = StringIO(out)
            blast_records = NCBIXML.parse(psiblast_xml)
            
            #~ blast_records = CommonFastaFunctions.runBlastParser(cline, blast_out_file)
            verboseprint("Blasted alleles on genome at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

            alleleSizes = []
            for allele in fullAlleleList:
                alleleSizes.append(len(allele))

            biggestSizeAllele = max(alleleSizes)

            # get mode allele size
            moda = max(set(alleleSizes), key=alleleSizes.count)
            contador = Counter(alleleSizes).most_common()

            # if most common allele size appears 1 time, get first allele size
            if (contador[0])[1] == 1:
                moda = alleleSizes[0]

            try:

                # iterate through the blast results
                for blast_record in blast_records:

                    locationcontigs = []

                    for alignment in blast_record.alignments:

                        # select the best match
                        for match in alignment.hsps:

                            # query id comes with query_id, not name of the allele
                            alleleMatchid = int((blast_record.query_id.split("_"))[-1])

                            # ~ scoreRatio=float(match.score)/float(allelescores[int(alleleMatchid)-1])
                            # query_id starts with 1
                            alleleMatchid2 = ((listShortAllelesNames[alleleMatchid - 1]).split("_"))[-1]
                            scoreRatio = float(match.score) / float(allelescores[int(alleleMatchid2)])

                            cdsStrName = (alignment.title.split(" "))[1]

                            #DNAstr = str(currentCDSDict[">" + cdsStrName])

                            AlleleDNAstr = alleleList[int(alleleMatchid) - 1]
                            verboseprint(str(match))
                            verboseprint("BSR : " + str(scoreRatio))

                            if scoreRatio >= bsrTresh:
                                locationcontigs.append(cdsStrName)

                            # select the best match from BLAST results

                            if scoreRatio == 1 and match.score > bestmatch[0]:
                                bestmatch = [match.score, scoreRatio, False, cdsStrName, int(alleleMatchid), match,
                                             len(AlleleDNAstr)]

                            elif (match.score > bestmatch[0] and scoreRatio >= bsrTresh and scoreRatio > bestmatch[
                                1] and bestmatch[2] is False):
                                bestmatch = [match.score, scoreRatio, False, cdsStrName, int(alleleMatchid), match,
                                             len(AlleleDNAstr)]

                verboseprint("Classifying the match at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

                # if no best match was found it's a Locus Not Found

                # check for ambiguious bases
                if not bestmatch[0] == 0:
                    alleleStr = currentCDSDict[">" + bestmatch[3]]
                    listFoundAmbiguities = []
                    listambiguousBases = ['K', 'M', 'R', 'Y', 'S', 'W', 'B', 'V', 'H', 'D', 'X', 'N', '-', '.']
                    listFoundAmbiguities = [e for e in listambiguousBases if e in alleleStr]

                if bestmatch[0] == 0 or len(listFoundAmbiguities) > 0:

                    ###################
                    # LOCUS NOT FOUND #
                    ###################
                    if bestmatch[0] == 0:
                        perfectMatchIdAllele.append('LNF')
                        perfectMatchIdAllele2.append('LNF')
                        verboseprint("Locus not found, no matches \n")
                    else:

                        perfectMatchIdAllele.append('LNF')
                        perfectMatchIdAllele2.append('LNF')
                        verboseprint("Locus has strange base \n")

                # if more than one BSR >0.6 in two different CDSs it's a Non Paralog Locus
                elif len(list(set(locationcontigs))) > 1:
                    verboseprint("NIPH", "")
                    perfectMatchIdAllele.append('NIPH')
                    perfectMatchIdAllele2.append('NIPH')
                    for elem in locationcontigs:
                        verboseprint(elem)


                # if match with BSR >0.6 and not equal DNA sequences
                else:

                    # load the contig info of the genome to a dictionary
                    #g_fp = HTSeq.FastaReader(genomeFile)
                    for contig in SeqIO.parse(genomeFile, "fasta", generic_dna):
                        currentGenomeDict[contig.id] = len(str(contig.seq.upper()))

                    match = bestmatch[5]
                    geneLen = bestmatch[6]
                    alleleStr = currentCDSDict[">" + bestmatch[3]]
                    contigname = bestmatch[3]

                    contigname = contigname.split("&")
                    matchLocation = contigname[2]
                    matchLocation = matchLocation.split("-")
                    matchLocation = [int(matchLocation[0]) + 1, matchLocation[1]]
                    contigname = contigname[0]

                    bestMatchContigLen = currentGenomeDict[contigname]

                    protSeq, alleleStr = translateSeq(alleleStr)
                    # get extra space to the right and left between the allele and match and check if it's still inside the contig

                    rightmatchAllele = geneLen - ((int(match.query_end) + 1) * 3)
                    leftmatchAllele = ((int(match.query_start) - 1) * 3)

                    
                    Reversed=False
                    # ~ if Reversed swap left and right contig extra
                    if int(matchLocation[1]) < int(matchLocation[0]):
                        rightmatchContig = bestMatchContigLen - int(matchLocation[0])
                        leftmatchContig = int(matchLocation[1])
                        aux = rightmatchAllele
                        rightmatchAllele = leftmatchAllele
                        leftmatchAllele = aux
                        Reversed=True

                    else:
                        rightmatchContig = bestMatchContigLen - int(matchLocation[1])
                        leftmatchContig = int(matchLocation[0])
                        

                    ###########################
                    # LOCUS ON THE CONTIG TIP #
                    ###########################

                    # check if contig is smaller than the matched allele
                    if leftmatchContig < leftmatchAllele and rightmatchContig < rightmatchAllele:

                        # ~ resultsList.append('PLOTSC:-1')
                        perfectMatchIdAllele.append('LOTSC')
                        perfectMatchIdAllele2.append('LOTSC')
                        # ~ if not Reversed:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"+")
                        # ~ else:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"-")

                        verboseprint(match, contigname, geneFile, leftmatchAllele, rightmatchAllele,
                                     "Locus is bigger than the contig \n")


                    elif leftmatchContig < leftmatchAllele:

                        # ~ resultsList.append('PLOT3:-1')
                        perfectMatchIdAllele.append('PLOT3')
                        perfectMatchIdAllele2.append('PLOT3')
                        # ~ if not Reversed:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"+")
                        # ~ else:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"-")

                        verboseprint(match, contigname, geneFile, leftmatchAllele, rightmatchAllele,
                                     "Locus is on the 3' tip of the contig \n")


                    elif rightmatchContig < rightmatchAllele:

                        # ~ resultsList.append('PLOT5:-1')
                        perfectMatchIdAllele.append('PLOT5')
                        perfectMatchIdAllele2.append('PLOT5')
                        # ~ if not Reversed:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"+")
                        # ~ else:
                        # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"-")

                        verboseprint(match, contigname, geneFile, leftmatchAllele, rightmatchAllele,
                                     "Locus is on the 5' tip of the contig \n")



                    elif float(len(alleleStr)) > moda + (moda * sizeTresh):

                        verboseprint("Locus is larger than mode", moda, alleleStr)

                        # ~ resultsList.append('ALM')
                        perfectMatchIdAllele.append('ALM')
                        perfectMatchIdAllele2.append('ALM')
                    # ~ if not Reversed:
                    # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"+")
                    # ~ else:
                    # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"-")

                    elif float(len(alleleStr)) < moda - (moda * sizeTresh):

                        verboseprint("Locus is smaller than mode", moda, alleleStr)

                        # ~ resultsList.append('ASM')
                        perfectMatchIdAllele.append('ASM')
                        perfectMatchIdAllele2.append('ASM')
                    # ~ if not Reversed:
                    # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"+")
                    # ~ else:
                    # ~ perfectMatchIdAllele2.append(str(contigname)+"&"+str(matchLocation[0])+"-"+str(matchLocation[1])+"&"+"-")


                    else:
                        #######################
                        # ADD INFERRED ALLELE #		# a new allele
                        #######################

                        wasContained = False
                        tagAuxC = 'S'
                        for alleleaux in fullAlleleList:

                            if alleleStr in alleleaux:
                                alleleName = fullAlleleNameList[fullAlleleList.index(alleleaux)]
                                alleleMatchid = (alleleName.split("_"))[-1]
                                tagAuxC = 'CD' + alleleMatchid.rstrip()
                                resultsList.append([(os.path.basename(genomeFile)), str(alleleI + 1), tagAuxC])
                                break
                            elif alleleaux in alleleStr:
                                alleleName = fullAlleleNameList[fullAlleleList.index(alleleaux)]
                                alleleMatchid = (alleleName.split("_"))[-1]
                                tagAuxC = 'CS' + alleleMatchid.rstrip()
                                resultsList.append([(os.path.basename(genomeFile)), str(alleleI + 1), tagAuxC])
                                break

                        if not wasContained:
                            tagAux = 'INF'

                            perfectMatchIdAllele.append(tagAux + "-" + str(alleleI + 1))

                            if not Reversed:
                                perfectMatchIdAllele2.append(str(contigname) + "&" + str(matchLocation[0]) + "-" + str(
                                    matchLocation[1]) + "&" + "+")
                            else:
                                perfectMatchIdAllele2.append(str(contigname) + "&" + str(matchLocation[0]) + "-" + str(
                                    matchLocation[1]) + "&" + "-")

                            verboseprint(
                                "New allele! Adding allele " + tagAux + str(alleleI + 1) + " to the database\n")

                            # --- add the new allele to the gene fasta --- #

                            alleleI += 1
                            appendAllele = '>' + str((((os.path.basename(geneFile)).split("."))[0]).replace("_",
                                                                                                            "-")) + "_" + tagAuxC + "_" + (
                                           str(os.path.basename(genomeFile))).replace("_", "-") +"_"+time.strftime("%d/%m/%YT%H:%M:%S") + "_" + str(
                                alleleI) + '\n'
                            
                            
                            #~ fG = open(geneFile, 'a')
                            #~ fG.write(appendAllele)
                            #~ fG.write(alleleStr + '\n')
                            #~ fG.close()
                            
                            newDNAAlleles2Add2Fasta+=appendAllele+alleleStr + '\n'
                            
                            fullAlleleList.append(alleleStr)
                            fullAlleleNameList.append(appendAllele)

                            if float(bestmatch[1]) >= bsrTresh and float(bestmatch[1]) < bsrTresh + 0.1:

                                newDNAAlleles2Add2shortFasta+=appendAllele+alleleStr + '\n'

                                geneTransalatedPath2 = os.path.join(basepath, str(
                                    os.path.basename(shortgeneFile) + '_protein2.fasta'))
                                geneTransalatedPath = os.path.join(basepath, str(
                                    os.path.basename(shortgeneFile) + '_protein.fasta'))



                                #~ proteinFastaString+='>' + alleleIaux + '\n' + str(protSeq) + '\n'
                                proteinFastaString+='>' + str(alleleI) + '\n' + str(protSeq) + '\n'
                                match = bestmatch[5]

                                # --- remake blast DB and recalculate the BSR for the locus --- #
                                alleleList.append(alleleStr)
                                listShortAllelesNames.append(appendAllele)

                                #~ sequence_2_blast='>' + alleleIaux + '\n' + str(protSeq)
                                sequence_2_blast='>' + str(alleleI) + '\n' + str(protSeq)
                                Gene_Blast_DB_name2 = CommonFastaFunctions.Create_Blastdb_no_fasta(geneTransalatedPath2, 1, True,sequence_2_blast)
                                
                                
                                verboseprint("Re-calculating BSR at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))
                                allelescores, alleleList, listShortAllelesNames = reDogetBlastScoreRatios(sequence_2_blast,
                                                                                                          basepath,
                                                                                                          alleleI,
                                                                                                          allelescores,
                                                                                                          Gene_Blast_DB_name2,
                                                                                                          alleleList,
                                                                                                          geneScorePickle,
                                                                                                          verbose,
                                                                                                          blastPath,
                                                                                                          listShortAllelesNames)
                                verboseprint("Done Re-calculating BSR at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))



            except Exception as e:
                print ("some error occurred")
                print (e)
                print ('Error on line {}'.format(sys.exc_info()[-1].tb_lineno))
                perfectMatchIdAllele2.append("ERROR")
                perfectMatchIdAllele.append("ERROR")

    #add new alleles to the locus fasta file
    
    if len(newDNAAlleles2Add2Fasta)>5:
        with open(geneFile, 'a') as fG:
            fG.write(newDNAAlleles2Add2Fasta)
    if len(newDNAAlleles2Add2shortFasta)>5:
        with open(shortgeneFile, 'a') as fG:
            fG.write(newDNAAlleles2Add2shortFasta)
    
    final = (resultsList, perfectMatchIdAllele)
    verboseprint("Finished allele calling at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))
    filepath = os.path.join(temppath, os.path.basename(geneFile) + "_result.txt")
    filepath2 = os.path.join(temppath, os.path.basename(geneFile) + "_result2.txt")
    with open(filepath, 'wb') as f:
        pickle.dump(final, f)
    with open(filepath2, 'wb') as f:
        pickle.dump(perfectMatchIdAllele2, f)
    shutil.rmtree(basepath)
    return True


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import sys
from Bio.Seq import Seq
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import os
import time
import pickle
import shutil
import multiprocessing
import subprocess
import json
import re
try:
    from allelecall import callAlleles_protein3,runProdigal,Create_Genome_Blastdb
    from utils import ParalogPrunning
except ImportError:
    from CHEWBBACA.allelecall import callAlleles_protein3,runProdigal,Create_Genome_Blastdb
    from CHEWBBACA.utils import ParalogPrunning

def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True

    print (program+" not found")
    sys.exit()
    return "Not found"


def prepGenomes(genomeFile, basepath, verbose,inputCDS):
    if verbose:
        def verboseprint(*args):
            for arg in args:
                print (arg),
            print()
    else:
        verboseprint = lambda *a: None  # do-nothing function

    listOfCDS = {}
    genomeProts = ""
    currentCDSDict = {}
    currentGenomeDict = {}

    j = 0
    if inputCDS==False:

        filepath = os.path.join(basepath, str(os.path.basename(genomeFile)) + "_ORF.txt")
        with open(filepath, 'rb') as f:
            currentCDSDict = pickle.load(f)

        for contig in SeqIO.parse(genomeFile, "fasta", generic_dna):
            sequence = str(contig.seq.upper())
            currentGenomeDict[contig.id] = sequence


        for contigTag, value in currentCDSDict.items():

            for protein in value:
                try:
                    seq = currentGenomeDict[contigTag][protein[0]:protein[1]].upper()
                    protseq, inverted, seq = translateSeq(seq, verbose)
                    j += 1
                    if inverted:
                        idstr = ">" + contigTag + "&protein" + str(j) + "&" + str(protein[1]) + "-" + str(protein[0])
                    else:
                        idstr = ">" + contigTag + "&protein" + str(j) + "&" + str(protein[0]) + "-" + str(protein[1])
                    genomeProts += idstr + "\n"
                    listOfCDS[idstr] = seq
                    genomeProts += str(protseq) + "\n"
                except Exception as e:
                    verboseprint((str(e) + " " + str(genomeFile)))
                    pass

    else:
        for contig in SeqIO.parse(genomeFile, "fasta", generic_dna):
            sequence = str(contig.seq.upper())
            currentGenomeDict[contig.id] = sequence
            try:
                protseq, inverted, seq = translateSeq(sequence, verbose)
                j += 1
                if inverted:
                    idstr = ">" + contig.id + "&protein" + str(j) + "&0-" + str(len(sequence))
                else:
                    idstr = ">" + contig.id + "&protein" + str(j) + "&0-" + str(len(sequence))
                genomeProts += idstr + "\n"
                listOfCDS[idstr] = seq
                genomeProts += str(protseq) + "\n"
            except:
                print (contig.id+" is not translatable to protein, sequence ignored")
                pass

    if j<2:
        raise ValueError("your genome has something wrong, are you using a genome as a CDS fasta file or vice versa?")

    filepath = os.path.join(basepath, str(os.path.basename(genomeFile)) + "_ORF_Protein.txt")
    with open(filepath, 'wb') as f:
        var = listOfCDS
        pickle.dump(var, f)
    listOfCDS = ''

    filepath = os.path.join(basepath, str(os.path.basename(genomeFile)) + "_Protein.fasta")
    with open(filepath, 'w') as f:
        f.write(genomeProts)
    genomeProts = ''
    var = ''
    currentGenomeDict = ''
    currentCDSDict = ''

    return True


def reverseComplement(strDNA):
    basecomplement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    strDNArevC = ''
    for l in strDNA:
        strDNArevC += basecomplement[l]

    return strDNArevC[::-1]


def translateSeq(DNASeq, verbose):
    if verbose:
        def verboseprint(*args):
            for arg in args:
                print (arg),
            print
    else:
        verboseprint = lambda *a: None  # do-nothing function

    seq = DNASeq
    tableid = 11
    inverted = False
    try:
        myseq = Seq(seq)
        protseq = Seq.translate(myseq, table=tableid, cds=True)
    except:
        try:
            seq = reverseComplement(seq)
            myseq = Seq(seq)
            protseq = Seq.translate(myseq, table=tableid, cds=True)
            inverted = True
        except:
            try:
                seq = seq[::-1]
                myseq = Seq(seq)
                protseq = Seq.translate(myseq, table=tableid, cds=True)
                inverted = True
            except:
                try:
                    seq = seq[::-1]
                    seq = reverseComplement(seq)
                    myseq = Seq(seq)
                    protseq = Seq.translate(myseq, table=tableid, cds=True)
                    inverted = False
                except Exception as e:
                    verboseprint("translation error")
                    verboseprint(e)
                    raise

    return protseq, inverted, str(seq)


def loci_translation(genesList, listOfGenomes2, verbose):
    gene_fp = open(genesList, 'r')

    genepath = ''
    basepath = ''
    lGenesFiles = []
    argumentsList = []
    noshortgeneFile = []

    for gene in gene_fp:
        k = 0
        gene = gene.rstrip('\n')
        multiple = True
        shortgene = os.path.join(os.path.dirname(gene), "short", os.path.basename(gene))
        shortgene = shortgene.replace(".fasta", "_short.fasta")
        if not os.path.isfile(shortgene):
            noshortgeneFile.append(gene)
            break

        #gene_fp2 = HTSeq.FastaReader(shortgene)
        for allele in SeqIO.parse(shortgene, "fasta", generic_dna):
            sequence=str(allele.seq.upper())
            k += 1
            if len(sequence) % 3 != 0:
                multiple = False
                print ("allele " + str(k) + " is not multiple of 3: " + str(gene) + " this gene is to be removed")
                break
            else:
                try:
                    protseq, Inverted, seq = translateSeq(sequence, verbose)
                except:
                    multiple = False
                    print ("allele " + str(k) + " is not translating : " + str(gene) + " this gene is to be removed")
                    break
        if multiple:
            lGenesFiles.append(gene)
            genepath = os.path.dirname(gene)
            basepath = os.path.join(genepath, "temp")
            if not os.path.exists(basepath):
                os.makedirs(basepath)
            filepath = os.path.join(basepath, str(os.path.basename(gene)) + "_argList.txt")
            with open(filepath, 'wb') as f:
                var = [gene, listOfGenomes2, genesList]
                pickle.dump(var, f)
            argumentsList.append(filepath)

    gene_fp.close()
    lGenesFiles.sort(key=lambda y: y.lower())
    return genepath, basepath, lGenesFiles, argumentsList, noshortgeneFile



# ================================================ MAIN ================================================ #

def main(genomeFiles,genes,cpuToUse,gOutFile,BSRTresh,BlastpPath,forceContinue,jsonReport,verbose,forceReset,contained,chosenTaxon,chosenTrainingFile,inputCDS,sizeTresh):

    #~ parser = argparse.ArgumentParser(description="This program call alleles for a set of genomes provided a schema")
    #~ parser.add_argument('-i', nargs='?', type=str, help='List of genome files (list of fasta files)', required=True)
    #~ parser.add_argument('-g', nargs='?', type=str, help='List of genes (fasta)', required=True)
    #~ parser.add_argument('-o', nargs='?', type=str, help="Name of the output files", required=True)
    #~ parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus, if over the maximum uses maximum -2",
    #~ required=True)
    #~ parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
    #~ default=False)
    #~ parser.add_argument('-b', nargs='?', type=str, help="BLAST full path", required=False, default='blastp')
    #~ parser.add_argument('--bsr', nargs='?', type=float, help="minimum BSR score", required=False, default=0.6)
    #~ parser.add_argument("--so", help="split the output per genome", dest='divideOutput', action="store_true",
    #~ default=False)
    #~ parser.add_argument('-t', nargs='?', type=str, help="taxon", required=False, default=False)
    #~ parser.add_argument('--ptf', nargs='?', type=str, help="provide own training file path", required=False, default=False)
    #~ parser.add_argument("--fc", help="force continue", required=False, action="store_true", default=False)
    #~ parser.add_argument("--fr", help="force reset", required=False, action="store_true", default=False)
    #~ parser.add_argument("--contained", help=argparse.SUPPRESS, required=False, action="store_true", default=False)
    #~ parser.add_argument("--json", help="report in json file", required=False, action="store_true", default=False)
    #~
    #~ args = parser.parse_args()
    #~
    #~ genomeFiles = args.i
    #~ genes = args.g
    #~ cpuToUse = args.cpu
    #~ BSRTresh = args.bsr
    #~ verbose = args.verbose
    #~ BlastpPath = args.b
    #~ divideOutput = args.divideOutput
    #~ gOutFile = args.o
    #~ chosenTaxon = args.t
    #~ chosenTrainingFile = args.ptf
    #~ forceContinue = args.fc
    #~ forceReset = args.fr
    #~ jsonReport = args.json
    #~ contained = args.contained
    divideOutput=False

    # avoid user to run the script with all cores available, could impossibilitate any usage when running on a laptop
    if cpuToUse > multiprocessing.cpu_count() - 2:
        print ("Warning, you are close to use all your cpus, if you are using a laptop you may be uncapable to perform any action")

    taxonList = {'Campylobacter jejuni': 'trained_campyJejuni.trn',
                 'Acinetobacter baumannii': 'trained_acinetoBaumannii.trn',
                 'Streptococcus agalactiae': 'trained_strepAgalactiae.trn',
                 'Haemophilus influenzae': 'trained_haemoInfluenzae_A.trn',
                 'Yersinia enterocolitica': 'trained_yersiniaEnterocolitica.trn',
                 'Escherichia coli': 'trained_eColi.trn',
                 'Enterococcus faecium': 'trained_enteroFaecium.trn',
                 'Staphylococcus haemolyticus': 'trained_staphHaemolyticus.trn',
                 'Salmonella enterica': 'trained_salmonellaEnterica_enteritidis.trn',
                 'Staphylococcus aureus': 'trained_StaphylococcusAureus.trn',
                 'Streptococcus pneumoniae': 'trained_strepPneumoniae.trn'
                 }
    if isinstance(chosenTaxon, str):
        trainingFolderPAth = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'TrainingFiles4Prodigal'))
        try:
            chosenTaxon = os.path.join(trainingFolderPAth, taxonList[chosenTaxon])

            if os.path.isfile(chosenTaxon):
                print ("will use this training file : " + chosenTaxon)
            else:
                print ("training file don't exist "+chosenTaxon)
                return "retry"
        except:
            print("Your chosen taxon is not attributed, select one from:")
            for elem in taxonList.keys():
                print (elem)
            return "retry"

    if isinstance(chosenTrainingFile, str):
        trainingFolderPAth = os.path.abspath(chosenTrainingFile)
        try:
            chosenTaxon = trainingFolderPAth

            if os.path.isfile(chosenTaxon):
                print ("will use this training file : " + chosenTaxon)
            else:
                print ("training file don't exist "+chosenTaxon)
                return "retry"
        except:
            print("The training file you provided doesn't exist:")
            print (chosenTaxon)
            return "retry"

    print(BlastpPath)

    scripts_path = os.path.dirname(os.path.realpath(__file__))

    print ("Will use this number of cpus: " + str(cpuToUse))
    print ("Checking all programs are installed")

    print ("Checking Blast installed... " + str(which(str(BlastpPath))))
    print ("Checking Prodigal installed... " + str(which('prodigal')))

    # check version of Blast

    proc = subprocess.Popen([BlastpPath, '-version'], stdout=subprocess.PIPE)
    stdout, stderr = proc.communicate()
    version_string = stdout.decode('utf8')
    blast_version_pat = re.compile(r'2.[5-9]')
    try:
        if not blast_version_pat.search(version_string):
            m = blast_version_pat.search(version_string).group()
            print ("your blast version is " + str(version_string))
            print ("update your blast to 2.5.0 or above, will exit program")
            sys.exit()
        else:
            print ("blast version is up to date, the program will continue")
    except:
        print("Something went wrong. Your blast version is " + str(version_string))
        print("update your blast to 2.5.0 or above, will exit program")

    starttime = "\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y")
    print (starttime)

    listOfGenomes = []
    listOfGenomesBasename = []

    print ("checking if genome files exist..")
    with open(genomeFiles, 'r') as fp:
        for genomeFile in fp:

            genomeFile = genomeFile.rstrip('\n')
            genomeFile = genomeFile.rstrip('\r')
            if os.path.isfile(genomeFile):
                listOfGenomes.append(genomeFile)
                listOfGenomesBasename.append(os.path.basename(genomeFile))
            else:
                print ("File does not exist, will not be used : " + str(genomeFile))

    if len(listOfGenomes) == 0:
        raise ValueError('ERROR! No usable genome files in ' + str(genomeFiles))

    # check if remnant files from previous run exist, prompt user if exists to know if it's his run and want to continue or start a new one

    lGenesFiles = []

    print ("checking if gene files exist..")
    with open(genes, 'r') as gene_fp:
        for gene in gene_fp:
            gene = gene.rstrip('\n')
            gene = gene.rstrip('\r')
            if os.path.isfile(gene):
                lGenesFiles.append(gene)
            else:
                print ("File does not exist, will not be used : " + str(gene))

    if len(lGenesFiles) == 0:
        raise ValueError('ERROR! No usable gene files in ' + str(genes))

    #sort the genomes and genes list for an ordered output
    listOfGenomes.sort(key=lambda y: os.path.basename(y).lower())
    listOfGenomesBasename.sort(key=lambda y: y.lower())
    lGenesFiles.sort(key=lambda y: y.lower())

    # create temp folder inside the folder where the first gene is located
    first_gene = lGenesFiles[0]
    genepath = os.path.dirname(first_gene)
    basepath = os.path.join(genepath, "temp")
    testVar = ""
    if os.path.isdir(basepath) and not forceContinue and not forceReset:
        testVar = input(
            "We found files belonging to a previous run not finished, If they are yours and want to continue were it stopped type Y or yes")
    continueRun = False

    if testVar.lower() == "yes" or testVar.lower() == "y" or forceContinue:
        if os.path.isdir(basepath):
            continueRun = True

        if forceReset:
            continueRun = True

    if continueRun:
        print ("You chose to continue the allele call")
        argumentsList = []
        resultsList = []
        for filee in os.listdir(basepath):
            if filee.endswith("_argList.txt"):
                argumentsList.append(os.path.join(basepath, str(filee)))
            elif filee.endswith("_result.txt"):
                resultsList.append((os.path.join(basepath, str(filee))).replace("_result.txt", "_argList.txt"))

        # remove unfinished directories
        todelFolders = next(os.walk(genepath))[1]
        for foldertodel in todelFolders:
            if "short" in foldertodel or "temp" in foldertodel:
                pass
            else:
                shutil.rmtree(os.path.join(genepath, str(foldertodel)))

    if not continueRun:

        try:
            # user decided not to continue although there was a run that had been stopped before finishing, files will be removed for a new run
            if os.path.isdir(basepath):
                print ("You chose to do a new allele call")
                print ("removing existing files...")
                shutil.rmtree(basepath)

            # translate the loci
            genepath, basepath, lGenesFiles, argumentsList, noShort = loci_translation(genes, listOfGenomes, verbose)

            # starting a fresh allele call process, going to run prodigal for all genomes on the list

            if len(argumentsList) == 0:
                shutil.rmtree(basepath)
                raise ValueError('ERROR! AT LEAST ONE GENE FILE MUST CONTAIN AN ALLELE REPRESENTING A CDS')

            if len(noShort) > 0:
                shutil.rmtree(basepath)
                raise ValueError('ERROR! These loci have no short gene file: ' + str(noShort))

            # ------------------------------------------------- #
            #           RUN PRODIGAL OVER ALL GENOMES           #
            # ------------------------------------------------- #

            #if the input is a draft genome
            if inputCDS==False:

                print ("\nStarting Prodigal at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

                # Prodigal run on the genomes, one genome per core using n-2 cores (n number of cores)

                pool = multiprocessing.Pool(cpuToUse)
                for genome in listOfGenomes:
                    pool.apply_async(runProdigal.main, (str(genome), basepath, str(chosenTaxon)))

                pool.close()
                pool.join()

                print ("\nChecking all prodigal processes created the necessary files...")

                listOfORFCreated = []
                for orffile in os.listdir(basepath):
                    if orffile.endswith("_ORF.txt"):
                        listOfORFCreated.append(orffile)

                if len(listOfGenomes) > len(listOfORFCreated):
                    message = "Missing some files from prodigal. " + str(
                        (len(listOfGenomes)) - (len(listOfORFCreated))) + " missing files out of " + str(len(listOfGenomes))
                    shutil.rmtree(basepath)
                    raise ValueError(message)
                else:
                    print ("All prodigal files necessary were created\n")

                print ("Finishing Prodigal at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))



            # ---CDS to protein---#

            # translate the genome CDSs, load them into dictionaries and fasta files to be used further ahead

            print ("Translating genomes")
            pool = multiprocessing.Pool(cpuToUse)
            for genomeFile in listOfGenomes:
                pool.apply_async(prepGenomes, args=[str(genomeFile), basepath, verbose,inputCDS])
            pool.close()
            pool.join()

            print ("Starting Genome Blast Db creation at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

            # creation of the Databases for each genome, one genome per core using n cores

            pool = multiprocessing.Pool(cpuToUse)
            for genomeFile in listOfGenomes:
                filepath = os.path.join(basepath, str(os.path.basename(genomeFile)) + "_Protein.fasta")
                os.makedirs(os.path.join(basepath, str(os.path.basename(genomeFile))))

                pool.apply_async(Create_Genome_Blastdb.main,(filepath,os.path.join(basepath, str(os.path.basename(genomeFile))),str(os.path.basename(genomeFile)),False))


            pool.close()
            pool.join()

        except Exception as e:
            exc_type, exc_obj, tb = sys.exc_info()
            lineno = tb.tb_lineno
            print (lineno)

            if jsonReport:
                runReport = {'finalStatus': 'error : ' + str(e)}
                with open('reportStatus.txt', 'w') as outfile:
                    json.dump(runReport, outfile)
            else:
                print (e)
            raise ValueError(e)
    else:

        # user decided to continue the stopped run, finding out which genes were left to run
        argumentsList = list(set(argumentsList) - set(resultsList))
        argumentsList = sorted(argumentsList)

    print
    print ("Starting Allele Calling at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    # Run the allele call, one gene per core using n cores


    pool = multiprocessing.Pool(cpuToUse)
    for argList in argumentsList:
        #~ print (argList)
        #~ asdasd
        pool.apply_async(callAlleles_protein3.main,(str(argList), basepath, str(BlastpPath),str(verbose),str(BSRTresh),str(sizeTresh)))

    pool.close()
    pool.join()

    print ("Finished Allele Calling at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))

    print ("Wrapping up the results")

    output = []
    output2 = []
    for gene in lGenesFiles:
        filepath = os.path.join(basepath, os.path.basename(gene) + "_result.txt")
        filepath2 = os.path.join(basepath, os.path.basename(gene) + "_result2.txt")
        with open(filepath, 'rb') as f:
            var = pickle.load(f)
            output.append(var)
        with open(filepath2, 'rb') as f:
            var = pickle.load(f)
            output2.append(var)

    # delete all temp files
    shutil.rmtree(basepath)

    numberOfLoci = len(output[0][1])

    print ("##################################################\n "+str(numberOfLoci)+" genomes used for "+str(len(output))+"loci" )
    numberexactmatches = 0
    for gene in output:
        for gAllele in gene[1]:
            try:
                int(gAllele)
                numberexactmatches += 1
            except:
                pass

    print ("\n used a bsr of : " + str(BSRTresh))
    print ("\n"+str(numberexactmatches)+" exact matches found out of "+ str (len(output[0][1]) * len(lGenesFiles)))
    print ("\n"+str(float((numberexactmatches * 100) / (numberOfLoci * len(lGenesFiles))))+" percent of exact matches \n##################################################")
    print ("\nWriting output files\n")

    # wrapping up the results into a matrix and save in a file
    try:
        phylovout = []
        phylovout2 = []
        genesnames = []
        statistics = []

        for gene in lGenesFiles:
            genename = gene.split("/")
            genename = genename[len(genename) - 1]
            genesnames.append(genename)
        gene = 0

        containedOutpWrite = ''
        for geneOut in output:
            genome = 0
            alleleschema = []

            while genome < numberOfLoci:

                genename = (geneOut[1][genome]).split("_")
                if len(genename) != 1:
                    alleleschema.append(genename[1])
                else:
                    alleleschema.append(genename[0])
                genome += 1

            phylovout.append(alleleschema)
            if len(geneOut[0]) > 0:
                containedOutpWrite += lGenesFiles[gene] + "\n"
            for contained2 in geneOut[0]:
                containedOutpWrite += contained2[0] + "\t" + contained2[1] + "-->" + contained2[2] + "\n"
            gene += 1

        for geneOut in output2:
            gene = 0
            alleleschema = []
            while gene < len(output2[0]):
                genename = (geneOut[gene])

                alleleschema.append(genename)
                gene += 1
            phylovout2.append(alleleschema)

        genome = 0

        genesHeader = "FILE" + "\t" + ('\t'.join(map(str, genesnames)))
        finalphylovinput = genesHeader
        finalphylovinput2 = genesHeader

        if divideOutput:
            allelesDict = {}
            contigDict = {}
            statsDict = {}

        while genome < len(listOfGenomes):
            auxList = []
            currentGenome = listOfGenomesBasename[genome]
            statsaux = [0] * 7  # EXC INF LNF PLOT NIPH ALM ASM
            finalphylovinput += "\n" + currentGenome + "\t"
            for gene in phylovout:

                val = str(gene[genome])
                auxList.append(val)
                if "INF" in val:
                    statsaux[1] += 1
                elif "LNF" in val:
                    statsaux[2] += 1
                elif "PLOT" in val:
                    statsaux[3] += 1
                elif "NIPH" in val:
                    statsaux[4] += 1
                elif "ALM" in val:
                    statsaux[5] += 1
                elif "ASM" in val:
                    statsaux[6] += 1
                else:
                    statsaux[0] += 1

            if divideOutput:
                allelesDict[currentGenome] = ('\t'.join(map(str, auxList)))
            finalphylovinput += ('\t'.join(map(str, auxList)))
            genome += 1
            statistics.append(statsaux)

        genome = 0
        while genome < len(listOfGenomes):
            auxList = []
            currentGenome = listOfGenomesBasename[genome]
            finalphylovinput2 += "\n" + currentGenome + "\t"
            for gene in phylovout2:
                val = str(gene[genome])
                auxList.append(val)

            if divideOutput:
                contigDict[currentGenome] = ('\t'.join(map(str, auxList)))
            finalphylovinput2 += ('\t'.join(map(str, auxList)))
            genome += 1

        statsHeader = 'Genome\tEXC\tINF\tLNF\tPLOT\tNIPH\tALM\tASM'
        statswrite = statsHeader
        genome = 0
        while genome < len(listOfGenomes):
            auxList = []
            currentGenome = listOfGenomesBasename[genome]
            statswrite += "\n" + currentGenome + "\t"
            for k in statistics[genome]:
                auxList.append(str(k))

            if divideOutput:
                statsDict[currentGenome] = ('\t'.join(map(str, auxList)))
            statswrite += ('\t'.join(map(str, auxList)))
            genome += 1

        if not os.path.exists(gOutFile):
            os.makedirs(gOutFile)
        # outputpath=os.path.dirname(gOutFile)
        outputfolder = os.path.join(gOutFile, "results_" + str(time.strftime("%Y%m%dT%H%M%S")))
        os.makedirs(outputfolder)
        print (statswrite)
        # ~ print
        # ~ print containedOutpWrite

        if jsonReport:
            runReport = {'finalStatus': 'success'}
            with open(os.path.join(outputfolder, "reportStatus.json"), 'w') as outfile:
                json.dump(runReport, outfile)

            aux = []
            runReport = {}
            for allelename in ((finalphylovinput.splitlines()[0]).split('\t'))[1:]:
                aux.append(allelename)
            runReport['header'] = aux
            # ~ print runReport
            for line in (finalphylovinput.splitlines())[1:]:
                aux2 = line.split('\t')
                genome = aux2[0]
                runReport[genome] = aux2[1:]

            with open(os.path.join(outputfolder, "results_alleles.json"), 'w') as outfile:
                json.dump(runReport, outfile)

            aux = []
            runReport = {}
            for allelename in ((statswrite.splitlines()[0]).split('\t'))[1:]:
                aux.append(allelename)
            runReport['header'] = aux
            # ~ print runReport
            for line in (statswrite.splitlines())[1:]:
                aux2 = line.split('\t')
                genome = aux2[0]
                runReport[genome] = aux2[1:]

            with open(os.path.join(outputfolder, "results_statistics.json"), 'w') as outfile:
                json.dump(runReport, outfile)

        elif not divideOutput:
            with open(os.path.join(outputfolder, "results_alleles.tsv"), 'w') as f:
                f.write(finalphylovinput)

            with open(os.path.join(outputfolder, "results_statistics.tsv"), 'w') as f:
                f.write(str(statswrite))

            with open(os.path.join(outputfolder, "results_contigsInfo.tsv"), 'w') as f:
                f.write(str(finalphylovinput2))
            if contained:
                with open(os.path.join(outputfolder, "results_contained.txt"), 'w') as f:
                    f.write(str(containedOutpWrite))
            with open(os.path.join(outputfolder, "logging_info.txt"), 'w') as f:
                f.write(starttime)
                f.write("\nFinished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))
                f.write("\nnumber of genomes: " + str(len(listOfGenomes)))
                f.write("\nnumber of loci: " + str(len(lGenesFiles)))
                f.write("\nused this number of cpus: " + str(cpuToUse))
                f.write("\nused a bsr of : " + str(BSRTresh))

            print ("checking the existance of paralog genes...")
            ParalogPrunning.main(os.path.join(outputfolder, "results_contigsInfo.tsv"), outputfolder)


        else:
            for genome in listOfGenomesBasename:
                currentGenome = os.path.splitext(genome)[0]
                perGenomeFolder = os.path.join(outputfolder, currentGenome)
                os.makedirs(perGenomeFolder)
                with open(os.path.join(perGenomeFolder, currentGenome + "_statistics.txt"), 'w') as f:
                    f.write(statsHeader + "\n")
                    f.write(genome)
                    f.write(statsDict[genome])
                with open(os.path.join(perGenomeFolder, currentGenome + "_contigsInfo.txt"), 'w') as f:
                    f.write(genesHeader + "\n")
                    f.write(genome)
                    f.write(contigDict[genome])
                with open(os.path.join(perGenomeFolder, currentGenome + "_alleles.txt"), 'w') as f:
                    f.write(genesHeader + "\n")
                    f.write(genome)
                    f.write(allelesDict[genome])

    except Exception as e:
        print (e)

        exc_type, exc_obj, tb = sys.exc_info()
        lineno = tb.tb_lineno
        print (lineno)
        if jsonReport:
            runReport = {'finalStatus': 'error : ' + str(e) + ' at line: ' + str(lineno)}
            with open(os.path.join(outputfolder, 'reportStatus.json'), 'w') as outfile:
                json.dump(runReport, outfile)
        else:
            print (e)
            print (lineno)

    print (starttime)
    print ("Finished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))


if __name__ == "__main__":
    main()
